@@ -1,5 +1,5 @@
 /* This file is part of the FaCT++ DL reasoner
-Copyright (C) 2003-2011 by Dmitry Tsarkov
+Copyright (C) 2003-2013 by Dmitry Tsarkov
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -20,14 +20,13 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 #include "tOntologyLoader.h"
 #include "tOntologyPrinterLISP.h"
 #include "AxiomSplitter.h"
-#include "ELFNormalizer.h"
-#include "ELFReasoner.h"
+#include "AtomicDecomposer.h"
 
-const char* ReasoningKernel :: Version = "1.5.3";
+const char* ReasoningKernel :: Version = "1.6.2";
 const char* ReasoningKernel :: SupportedDL = "SROIQ(D)";
 const char* ReasoningKernel :: Copyright =
-	"Copyright (C) Dmitry Tsarkov, 2002-2011";
-const char* ReasoningKernel :: ReleaseDate = "7 December 2011";
+	"Copyright (C) Dmitry Tsarkov, 2002-2013";
+const char* ReasoningKernel :: ReleaseDate = "19 February 2013";
 
 // print the FaCT++ information only once
 static bool KernelFirstRun = true;
@@ -41,7 +40,11 @@ static bool KernelFirstRun = true;
 ReasoningKernel :: ReasoningKernel ( void )
 	: pTBox (NULL)
 	, pET(NULL)
-	, D2I(NULL)
+	, KE(NULL)
+	, AD(NULL)
+	, ModSyn(NULL)
+	, ModSem(NULL)
+	, JNICache(NULL)
 	, pMonitor(NULL)
 	, OpTimeout(0)
 	, verboseOutput(false)
@@ -49,7 +52,7 @@ ReasoningKernel :: ReasoningKernel ( void )
 	, cachedQuery(NULL)
 	, cachedQueryTree(NULL)
 	, useAxiomSplitting(false)
-	, useELReasoner(false)
+	, ignoreExprCache(false)
 {
 	// Intro
 	if ( KernelFirstRun )
@@ -66,6 +69,34 @@ ReasoningKernel :: ReasoningKernel ( void )
 		throw EFaCTPlusPlus("FaCT++ kernel: Cannot init options");
 }
 
+/// d'tor
+ReasoningKernel :: ~ReasoningKernel ( void )
+{
+	clearTBox();
+	delete cachedQueryTree;
+	delete pMonitor;
+}
+
+/// clear TBox and related structures; keep ontology in place
+void
+ReasoningKernel :: clearTBox ( void )
+{
+	delete pTBox;
+	pTBox = NULL;
+	delete pET;
+	pET = NULL;
+	delete KE;
+	KE = NULL;
+	delete AD;
+	AD = NULL;
+	delete ModSem;
+	ModSem = NULL;
+	delete ModSyn;
+	ModSyn = NULL;
+	// during preprocessing the TBox names were cached. clear that cache now.
+	getExpressionManager()->clearNameCache();
+}
+
 bool
 ReasoningKernel :: tryIncremental ( void )
 {
@@ -92,26 +123,7 @@ ReasoningKernel :: forceReload ( void )
 	// so switch it off after the 1st usage
 	pMonitor = NULL;
 
-	if ( useELReasoner )
-	{
-		ELFAxiomChecker ac;
-		ac.visitOntology(Ontology);
-		if ( ac )
-		{
-			std::cerr << "Normalizing EL ontology... ";
-			ELFNormalizer normalizer (getExpressionManager());
-			normalizer.visitOntology(Ontology);
-			std::cerr << "done\nLoading EL ontology... ";
-			ELFReasoner reasoner (Ontology);
-			std::cerr << "done\nClassifying EL ontology... ";
-			reasoner.classify();
-			std::cerr << "done\n";
-			exit(0);
-		}
-	}
-
 	// split ontological axioms
-
 	if ( useAxiomSplitting )
 	{
 		TAxiomSplitter AxiomSplitter(&Ontology);
@@ -254,10 +266,7 @@ ReasoningKernel :: setUpCache ( DLTree* query, cacheStatus level )
 	if ( isCN(cachedQueryTree) )
 		cachedConcept = getTBox()->getCI(cachedQueryTree);
 	else	// case of complex query
-	{
-		getTBox()->clearQueryConcept();
 		cachedConcept = getTBox()->createQueryConcept(cachedQueryTree);
-	}
 
 	fpp_assert ( cachedConcept != NULL );
 	// preprocess concept is necessary (fresh concept in query or complex one)
@@ -301,12 +310,7 @@ ReasoningKernel :: setUpCache ( TConceptExpr* query, cacheStatus level )
 	if ( isNameOrConst(cachedQuery) )
 		cachedConcept = getTBox()->getCI(TreeDeleter(e(cachedQuery)));
 	else	// case of complex query
-	{
-		// need to clear the query before transform it into DLTree
-		getTBox()->clearQueryConcept();
-		// ... as if fresh names appears there, they would be cleaned up
 		cachedConcept = getTBox()->createQueryConcept(TreeDeleter(e(cachedQuery)));
-	}
 
 	fpp_assert ( cachedConcept != NULL );
 	// preprocess concept is necessary (fresh concept in query or complex one)
@@ -355,6 +359,9 @@ ReasoningKernel :: checkSub ( TConcept* C, TConcept* D )
 			// C [= D iff D = \top, or ~D = \bottom
 			return !checkSatTree(createSNFNot(getTBox()->getTree(C)));
 	// here C and D are known (not fresh)
+	// check the obvious ones
+	if ( unlikely(D->isTop()) || unlikely(C->isBottom()) )
+		return true;
 	if ( getStatus() < kbClassified )	// unclassified => do via SAT test
 		return getTBox()->isSubHolds ( C, D );
 	// classified => do the taxonomy traversal
@@ -377,32 +384,41 @@ ReasoningKernel :: isDisjointRoles ( void )
 	TExprVec Disj = getExpressionManager()->getArgList();
 	TRoleVec Roles;
 	Roles.reserve(Disj.size());
+	unsigned int nTopRoles = 0;
 
 	for ( TExprVec::const_iterator p = Disj.begin(), p_end = Disj.end(); p != p_end; ++p )
 	{
-		TORoleExpr* ORole = dynamic_cast<TORoleExpr*>(*p);
-		if ( ORole != NULL )
+		if ( TORoleExpr* ORole = dynamic_cast<TORoleExpr*>(*p) )
 		{
-			if ( getExpressionManager()->isUniversalRole(ORole) )
-				return false;	// universal role is not disjoint with anything
-			if ( getExpressionManager()->isEmptyRole(ORole) )
+			TRole* R = getRole ( ORole, "Role expression expected in isDisjointRoles()" );
+			if ( R->isBottom() )
 				continue;		// empty role is disjoint with everything
-
-			Roles.push_back ( getRole ( ORole, "Role expression expected in isDisjointRoles()" ) );
+			if ( R->isTop() )
+				++nTopRoles;	// count universal roles
+			else
+				Roles.push_back(R);
+		}
+		else if ( TDRoleExpr* DRole = dynamic_cast<TDRoleExpr*>(*p) )
+		{
+			TRole* R = getRole ( DRole, "Role expression expected in isDisjointRoles()" );
+			if ( R->isBottom() )
+				continue;		// empty role is disjoint with everything
+			if ( R->isTop() )
+				++nTopRoles;	// count universal roles
+			else
+				Roles.push_back(R);
 		}
 		else
-		{
-			TDRoleExpr* DRole = dynamic_cast<TDRoleExpr*>(*p);
-			if ( DRole == NULL )
-				throw EFaCTPlusPlus("Role expression expected in isDisjointRoles()");
+			throw EFaCTPlusPlus ( "Role expression expected in isDisjointRoles()" );
+	}
 
-			if ( getExpressionManager()->isUniversalRole(DRole) )
-				return false;	// universal role is not disjoint with anything
-			if ( getExpressionManager()->isEmptyRole(DRole) )
-				continue;		// empty role is disjoint with everything
-
-			Roles.push_back ( getRole ( DRole, "Role expression expected in isDisjointRoles()" ) );
-		}
+	// deal with top-roles
+	if ( nTopRoles > 0 )
+	{
+		if ( nTopRoles > 1 || !Roles.empty() )
+			return false;	// universal role is not disjoint with anything but the bottom role
+		else
+			return true;
 	}
 
 	// test pair-wise disjointness
@@ -531,58 +547,101 @@ ReasoningKernel :: isRelated ( const TIndividualExpr* I, const TORoleExpr* R, co
 }
 
 //----------------------------------------------------------------------------------
-// knowledge exploration queries
+// atomic decomposition queries
 //----------------------------------------------------------------------------------
 
-/// build the set of data neighbours of a NODE, put the set into the RESULT variable
-void
-ReasoningKernel :: getDataRoles ( const TCGNode* node, TCGRoleSet& Result, bool onlyDet )
+	/// create new atomic decomposition of the loaded ontology using TYPE. @return size of the AD
+unsigned int
+ReasoningKernel :: getAtomicDecompositionSize ( bool useSemantic, ModuleType moduleType )
 {
-	Result.clear();
-	for ( DlCompletionTree::const_edge_iterator p = node->begin(), p_end = node->end(); p != p_end; ++p )
-		if ( likely(!(*p)->isIBlocked()) && (*p)->getArcEnd()->isDataNode() && (!onlyDet || (*p)->getDep().empty()) )
-			// FIXME!! add also all supers
-			Result.insert(getExpressionManager()->DataRole((*p)->getRole()->getName()));
+	// init AD field
+	if ( unlikely(AD != NULL) )
+		delete AD;
+
+	AD = new AtomicDecomposer(getModExtractor(useSemantic));
+	return AD->getAOS ( &Ontology, moduleType )->size();
 }
-/// build the set of object neighbours of a NODE; incoming edges are counted iff NEEDINCOMING is true
-void
-ReasoningKernel :: getObjectRoles ( const TCGNode* node, TCGRoleSet& Result, bool onlyDet, bool needIncoming )
+	/// get a set of axioms that corresponds to the atom with the id INDEX
+const TOntologyAtom::AxiomSet&
+ReasoningKernel :: getAtomAxioms ( unsigned int index ) const
 {
-	Result.clear();
-	for ( DlCompletionTree::const_edge_iterator p = node->begin(), p_end = node->end(); p != p_end; ++p )
-		if ( likely(!(*p)->isIBlocked()) && !(*p)->getArcEnd()->isDataNode() && (!onlyDet || (*p)->getDep().empty()) && (needIncoming || (*p)->isSuccEdge() ) )
-			Result.insert(getExpressionManager()->ObjectRole((*p)->getRole()->getName()));
+	return (*AD->getAOS())[index]->getAtomAxioms();
 }
-/// build the set of neighbours of a NODE via role ROLE; put the resulting list into RESULT
-void
-ReasoningKernel :: getNeighbours ( const TCGNode* node, TRoleExpr* role, TCGNodeVec& Result )
+	/// get a set of axioms that corresponds to the module of the atom with the id INDEX
+const TOntologyAtom::AxiomSet&
+ReasoningKernel :: getAtomModule ( unsigned int index ) const
 {
-	Result.clear();
-	TRole* R = getRole ( role, "Role expression expected in getNeighbours() method" );
-	for ( DlCompletionTree::const_edge_iterator p = node->begin(), p_end = node->end(); p != p_end; ++p )
-		if ( likely(!(*p)->isIBlocked()) && (*p)->isNeighbour(R) )
-			Result.push_back((*p)->getArcEnd());
+	return (*AD->getAOS())[index]->getModule();
 }
-/// put into RESULT all the data expressions from the NODE label
-void
-ReasoningKernel :: getLabel ( const TCGNode* node, TCGItemVec& Result, bool onlyDet )
+	/// get a set of atoms on which atom with index INDEX depends
+const TOntologyAtom::AtomSet&
+ReasoningKernel :: getAtomDependents ( unsigned int index ) const
 {
-	// prepare D2I translator
-	if ( unlikely(D2I == NULL) )
-		D2I = new TDag2Interface ( getTBox()->getDag(), getExpressionManager() );
-	else
-		D2I->ensureDagSize();
-	DlCompletionTree::const_label_iterator p, p_end;
-	bool data = node->isDataNode();
-	Result.clear();
-	for ( p = node->beginl_sc(), p_end = node->endl_sc(); p != p_end; ++p )
-		if ( !onlyDet || p->getDep().empty() )
-			Result.push_back(D2I->getExpr(p->bp(),data));
-	for ( p = node->beginl_cc(), p_end = node->endl_cc(); p != p_end; ++p )
-		if ( !onlyDet || p->getDep().empty() )
-			Result.push_back(D2I->getExpr(p->bp(),data));
+	return (*AD->getAOS())[index]->getDepAtoms();
 }
 
+TModularizer*
+ReasoningKernel :: getModExtractor ( bool useSemantic )
+{
+	bool needInit = false;
+	// check whether we need init
+	if ( useSemantic && unlikely(ModSem == NULL) )
+	{
+		ModSem = new TModularizer(/*useSem=*/true);
+		needInit = true;
+	}
+	if ( !useSemantic && unlikely(ModSyn == NULL) )
+	{
+		ModSyn = new TModularizer(/*useSem=*/false);
+		needInit = true;
+	}
+
+	// init if necessary
+	TModularizer* Mod = useSemantic ? ModSem : ModSyn;
+	if ( unlikely(needInit) )
+		Mod->preprocessOntology(getOntology().getAxioms());
+	return Mod;
+}
+
+/// get a set of axioms that corresponds to the atom with the id INDEX
+const AxiomVec&
+ReasoningKernel :: getModule ( bool useSemantic, ModuleType moduleType )
+{
+	// init signature
+	TSignature Sig;
+	Sig.setLocality(false);
+	const std::vector<const TDLExpression*> signature = getExpressionManager()->getArgList();
+	for ( std::vector<const TDLExpression*>::const_iterator q = signature.begin(), q_end = signature.end(); q != q_end; ++q )
+		if ( const TNamedEntity* entity = dynamic_cast<const TNamedEntity*>(*q) )
+			Sig.add(entity);
+	TModularizer* Mod = getModExtractor(useSemantic);
+	Mod->extract ( getOntology(), Sig, moduleType );
+	return Mod->getModule();
+}
+
+/// get a set of axioms that corresponds to the atom with the id INDEX
+const AxiomVec&
+ReasoningKernel :: getNonLocal ( bool useSemantic, ModuleType moduleType )
+{
+	// init signature
+	TSignature Sig;
+	Sig.setLocality(moduleType == M_TOP);	// true for TOP, false for BOT/STAR
+	const std::vector<const TDLExpression*> signature = getExpressionManager()->getArgList();
+	for ( std::vector<const TDLExpression*>::const_iterator q = signature.begin(), q_end = signature.end(); q != q_end; ++q )
+		if ( const TNamedEntity* entity = dynamic_cast<const TNamedEntity*>(*q) )
+			Sig.add(entity);
+
+	// do check
+	LocalityChecker* LC = getModExtractor(useSemantic)->getLocalityChecker();
+	LC->setSignatureValue(Sig);
+	Result.clear();
+	for ( TOntology::iterator p = getOntology().begin(), p_end = getOntology().end(); p != p_end; ++p )
+		if ( !LC->local(*p) )
+			Result.push_back(*p);
+	return Result;
+}
+
+
 //******************************************
 //* Initialization
 //******************************************
@@ -756,6 +815,26 @@ bool ReasoningKernel :: initOptions ( void )
 		) )
 		return true;
 
+	// options for testing
+
+	// register "checkAD" option (24/02/2012)
+	if ( KernelOptions.RegisterOption (
+		"checkAD",
+		"Option 'checkAD' forces FaCT++ to create the AD and exit instead of performing classification",
+		ifOption::iotBool,
+		"false"
+		) )
+		return true;
+
+	// register "useELReasoner" option (24/02/2012)
+	if ( KernelOptions.RegisterOption (
+		"useELReasoner",
+		"Option 'useELReasoner' forces FaCT++ to use the EL reasoner and exit instead of performing classification",
+		ifOption::iotBool,
+		"false"
+		) )
+		return true;
+
 	// all was registered OK
 	return false;
 }
