@@ -1,5 +1,5 @@
 /* This file is part of the FaCT++ DL reasoner
-Copyright (C) 2003-2011 by Dmitry Tsarkov
+Copyright (C) 2003-2013 by Dmitry Tsarkov
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -23,14 +23,18 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
 #include "fpp_assert.h"
 #include "eFPPInconsistentKB.h"
-#include "tNAryQueue.h"
 #include "dlTBox.h"
-#include "ReasonerNom.h"
 #include "ifOptions.h"
 #include "DLConceptTaxonomy.h"	// for getRelatives()
 #include "tExpressionTranslator.h"
 #include "tOntology.h"
-#include "tDag2Interface.h"
+#include "KnowledgeExplorer.h"
+#include "tOntologyAtom.h"	// types for AD
+#include "ModuleType.h"
+
+class TModularizer;
+class AtomicDecomposer;
+class TJNICache;	// cached JNI information
 
 class ReasoningKernel
 {
@@ -90,13 +94,17 @@ public:	// types interface
 	// types for knowledge exploration
 
 		/// type for the node in the completion graph
-	typedef const DlCompletionTree TCGNode;
+	typedef KnowledgeExplorer::TCGNode TCGNode;
 		/// type for the node vector
-	typedef std::vector<TCGNode*> TCGNodeVec;
+	typedef KnowledgeExplorer::TCGNodeVec TCGNodeVec;
 		/// type for a set of role expressions (used in KE to return stuff)
-	typedef std::set<TRoleExpr*> TCGRoleSet;
+	typedef KnowledgeExplorer::TCGRoleSet TCGRoleSet;
 		/// type for a vector of data/concept expressions (used in KE to return stuff)
-	typedef std::vector<TExpr*> TCGItemVec;
+	typedef KnowledgeExplorer::TCGItemVec TCGItemVec;
+
+protected:	// types
+		/// arguments for property chain querying and friends
+	typedef const std::vector<const TDLExpression*> TExprVec;
 
 private:
 		/// options for the kernel and all related substructures
@@ -134,9 +142,19 @@ protected:	// members
 		/// expression translator to work with queries
 	TExpressionTranslator* pET;
 		/// trace vector for the last operation (set from the TBox trace-sets)
-	std::vector<TDLAxiom*> TraceVec;
-		/// dag-2-interface translator used in knowledge exploration
-	TDag2Interface* D2I;
+	AxiomVec TraceVec;
+		/// knowledge exploration support
+	KnowledgeExplorer* KE;
+		/// atomic decomposer
+	AtomicDecomposer* AD;
+		/// syntactic locality based module extractor
+	TModularizer* ModSyn;
+		/// semantic locality based module extractor
+	TModularizer* ModSem;
+		/// set to return by the locality checking procedure
+	AxiomVec Result;
+		/// JNI cache corresponding to a kernel. External, created and deleted outside
+	TJNICache* JNICache;
 
 	// Top/Bottom role names: if set, they will appear in all hierarchy-related output
 
@@ -181,8 +199,8 @@ protected:	// members
 	bool NeedTracing;
 		/// whether axiom splitting should be used
 	bool useAxiomSplitting;
-		/// whether EL polynomial reasoner should be used
-	bool useELReasoner;
+		/// ignore cache for the TExpr* (useful for semantic AD)
+	bool ignoreExprCache;
 
 private:	// no copy
 		/// no copy c'tor
@@ -216,9 +234,9 @@ protected:	// methods
 		return *pET;
 	}
 		/// get fresh filled depending of a type of R
-	DLTree* getFreshFiller ( const DLTree* R )
+	DLTree* getFreshFiller ( const TRole* R )
 	{
-		if ( resolveRole(R)->isDataRole() )
+		if ( R->isDataRole() )
 			return getTBox()->getDataTypeCenter().getFreshDataType();
 		else
 			return getTBox()->getFreshConcept();
@@ -235,7 +253,7 @@ protected:	// methods
 		/// set query cache value to QUERY
 	void setQueryCache ( DLTree* query ) { clearQueryCache(); cachedQueryTree = query; }
 		/// check whether query cache is the same as QUERY
-	bool checkQueryCache ( TConceptExpr* query ) const { return cachedQuery == query; }
+	bool checkQueryCache ( TConceptExpr* query ) const { return ignoreExprCache ? false : cachedQuery == query; }
 		/// check whether query cache is the same as QUERY
 	bool checkQueryCache ( DLTree* query ) const { return equalTrees ( cachedQueryTree, query ); }
 		/// classify query; cache is ready at the point. NAMED means whether concept is just a name
@@ -265,6 +283,15 @@ protected:	// methods
 		return I->getRelatedCache(R);
 	}
 
+		/// get role expression based on the R
+	TDLRoleExpression* Role ( const TRole* R )
+	{
+		if ( unlikely(R->isDataRole()) )
+			return getExpressionManager()->DataRole(R->getName());
+		else
+			return getExpressionManager()->ObjectRole(R->getName());
+	}
+
 	//-----------------------------------------------------------------------------
 	//--		internal reasoning methods
 	//-----------------------------------------------------------------------------
@@ -272,6 +299,16 @@ protected:	// methods
 		/// @return true iff C is satisfiable
 	bool checkSatTree ( DLTree* C )
 	{
+		if ( unlikely(C->Element().getToken() == TOP) )
+		{
+			deleteTree(C);
+			return true;
+		}
+		if ( unlikely(C->Element().getToken() == BOTTOM) )
+		{
+			deleteTree(C);
+			return false;
+		}
 		setUpCache ( C, csSat );
 		return getTBox()->isSatisfiable(cachedConcept);
 	}
@@ -294,18 +331,18 @@ protected:	// methods
 	// helper methods to query properties of roles
 
 		/// @return true if R is functional wrt ontology
-	bool checkFunctionality ( DLTree* R )
+	bool checkFunctionality ( TRole* R )
 	{
 		// R is transitive iff \ER.C and \ER.\not C is unsatisfiable
-		DLTree* tmp = createSNFExists ( clone(R), createSNFNot(getFreshFiller(R)) );
-		tmp = createSNFAnd ( tmp, createSNFExists ( R, getFreshFiller(R) ) );
+		DLTree* tmp = createSNFExists ( createRole(R), createSNFNot(getFreshFiller(R)) );
+		tmp = createSNFAnd ( tmp, createSNFExists ( createRole(R), getFreshFiller(R) ) );
 		return !checkSatTree(tmp);
 	}
 		/// @return true if R is functional; set the value for R if necessary
 	bool getFunctionality ( TRole* R )
 	{
 		if ( !R->isFunctionalityKnown() )	// calculate functionality
-			R->setFunctional ( checkFunctionality ( new DLTree ( TLexeme ( R->isDataRole() ? DNAME : RNAME, R ) ) ) );
+			R->setFunctional(checkFunctionality(R));
 		return R->isFunctional();
 	}
 		/// @return true if R is transitive wrt ontology
@@ -334,32 +371,35 @@ protected:	// methods
 		return !checkSatTree(tmp);
 	}
 		/// @return true if R [= S wrt ontology
-	bool checkRoleSubsumption ( DLTree* R, DLTree* S )
+	bool checkRoleSubsumption ( TRole* R, TRole* S )
 	{
-		if ( resolveRole(R)->isDataRole() != resolveRole(S)->isDataRole() )
+		if ( unlikely ( R->isDataRole() != S->isDataRole() ) )
 			return false;
 		// R [= S iff \ER.C and \AS.(not C) is unsatisfiable
-		DLTree* tmp = createSNFForall ( S, createSNFNot(getFreshFiller(S)) );
-		tmp = createSNFAnd ( createSNFExists ( R, getFreshFiller(R) ), tmp );
+		DLTree* tmp = createSNFForall ( createRole(S), createSNFNot(getFreshFiller(S)) );
+		tmp = createSNFAnd ( createSNFExists ( createRole(R), getFreshFiller(R) ), tmp );
 		return !checkSatTree(tmp);
 	}
 		/// @return true iff the chain contained in the arg-list is a sub-property of R
-	bool checkSubChain ( TRole* R )
+	bool checkSubChain ( const TExprVec& Chain, TRole* R )
 	{
-		// retrieve a role chain
-		typedef const std::vector<const TDLExpression*> TExprVec;
-		TExprVec Chain = getExpressionManager()->getArgList();
-
 		// R1 o ... o Rn [= R iff \ER1.\ER2....\ERn.(notC) and AR.C is unsatisfiable
 		DLTree* tmp = createSNFNot(getTBox()->getFreshConcept());
 		for ( TExprVec::const_reverse_iterator p = Chain.rbegin(), p_end = Chain.rend(); p != p_end; ++p )
-		{
-			TORoleExpr* Ri = dynamic_cast<TORoleExpr*>(*p);
-			if ( Ri == NULL )
+			if ( TORoleExpr* Ri = dynamic_cast<TORoleExpr*>(*p) )
+			{
+				TRole* S = getRole ( Ri, "Role expression expected in chain of isSubChain()" );
+				if ( unlikely(S->isBottom()) )	// bottom in a chain makes it super of any role
+				{
+					deleteTree(tmp);
+					return true;
+				}
+				tmp = createSNFExists ( createRole(S), tmp );
+			}
+			else
 				throw EFaCTPlusPlus("Role expression expected in the role chain construct");
-			tmp = createSNFExists ( e(Ri), tmp );
-		}
-		tmp = createSNFAnd ( tmp, createSNFForall ( new DLTree(TLexeme(RNAME,R)), getTBox()->getFreshConcept() ) );
+
+		tmp = createSNFAnd ( tmp, createSNFForall ( createEntry(RNAME,R), getTBox()->getFreshConcept() ) );
 		return !checkSatTree(tmp);
 	}
 
@@ -376,15 +416,7 @@ protected:	// methods
 		/// get RO access to TBox
 	const TBox* getTBox ( void ) const { checkTBox(); return pTBox; }
 		/// clear TBox and related structures; keep ontology in place
-	void clearTBox ( void )
-	{
-		delete pTBox;
-		pTBox = NULL;
-		delete pET;
-		pET = NULL;
-		delete D2I;
-		D2I = NULL;
-	}
+	void clearTBox ( void );
 
 		/// get RW access to Object RoleMaster from TBox
 	RoleMaster* getORM ( void ) { return getTBox()->getORM(); }
@@ -422,7 +454,7 @@ protected:	// methods
 		/// get individual by the TIndividualExpr
 	TIndividual* getIndividual ( const TIndividualExpr* i, const char* reason )
 	{
-		try { return static_cast<TIndividual*>(getTBox()->getCI(e(i))); }
+		try { return static_cast<TIndividual*>(getTBox()->getCI(TreeDeleter(e(i)))); }
 		catch(...) { throw EFaCTPlusPlus(reason); }
 	}
 		/// get role by the TRoleExpr
@@ -466,15 +498,20 @@ protected:	// methods
 		/// load the status of the KB and the appropriate part of KB
 	void LoadKB ( std::istream& i );
 
+	//----------------------------------------------------------------------------------
+	// knowledge exploration queries
+	//----------------------------------------------------------------------------------
+
+		/// add the role R and all its supers to a set RESULT
+	void addRoleWithSupers ( const TRole* R, TCGRoleSet& Result );
+		/// check whether the modularizer need initialisation, init it and return a proper one
+	TModularizer* getModExtractor ( bool useSemantic );
+
 public:	// general staff
 		/// default c'tor
 	ReasoningKernel ( void );
 		/// d'tor
-	~ReasoningKernel ( void )
-	{
-		clearTBox();
-		delete pMonitor;
-	}
+	~ReasoningKernel ( void );
 
 	ifOptionSet* getOptions ( void ) { return &KernelOptions; }
 	const ifOptionSet* getOptions ( void ) const { return &KernelOptions; }
@@ -538,6 +575,10 @@ public:	// general staff
 	}
 		/// choose whether axiom splitting should be used
 	void setAxiomSplitting ( bool value ) { useAxiomSplitting = value; }
+		/// choose whether TExpr cache should be ignored
+	void setIgnoreExprCache ( bool value ) { ignoreExprCache = value; }
+		/// set the signature of the expression translator
+	void setSignature ( const TSignature* sig ) { if ( pET != NULL ) pET->setSignature(sig); }
 
 	//----------------------------------------------
 	//-- Tracing support
@@ -546,7 +587,7 @@ public:	// general staff
 		/// tells reasoner that the next reasoning operation shall gather the trace
 	void needTracing ( void ) { NeedTracing = true; }
 		/// @return the trace-set of the last reasoning operation
-	const std::vector<TDLAxiom*>& getTrace ( void )
+	const AxiomVec& getTrace ( void )
 	{
 		TraceVec.clear();
 		return TraceVec;
@@ -567,6 +608,15 @@ public:	// general staff
 
 		/// get access to an expression manager
 	TExpressionManager* getExpressionManager ( void ) { return Ontology.getExpressionManager(); }
+		/// get RW access to the ontology
+	TOntology& getOntology ( void ) { return Ontology; }
+		/// get RO access to the ontology
+	const TOntology& getOntology ( void ) const { return Ontology; }
+
+		/// set JNI cache
+	void setJNICache ( TJNICache* cache ) { JNICache = cache; }
+		/// get JNI cache
+	TJNICache* getJNICache ( void ) const { return JNICache; }
 
 public:
 	//******************************************
@@ -735,9 +785,13 @@ public:
 		/// return consistency status of KB
 	bool isKBConsistent ( void )
 	{
-		if ( getStatus() <= kbLoading )
-			processKB(kbCChecked);
-		return getTBox()->isConsistent();
+		try {
+			if ( getStatus() <= kbLoading )
+				processKB(kbCChecked);
+			return getTBox()->isConsistent();
+		}
+		catch ( const EFPPInconsistentKB& )
+			{ return false; }
 	}
 		/// ensure that KB is preprocessed/consistence checked
 	void preprocessKB ( void )
@@ -768,45 +822,48 @@ public:
 	bool isFunctional ( const TORoleExpr* R )
 	{
 		preprocessKB();	// ensure KB is ready to answer the query
-		if ( getExpressionManager()->isUniversalRole(R) )
+		TRole* r = getRole ( R, "Role expression expected in isFunctional()" );
+		if ( unlikely(r->isTop()) )
 			return false;	// universal role is not functional
-		if ( getExpressionManager()->isEmptyRole(R) )
+		if ( unlikely(r->isBottom()) )
 			return true;	// empty role is functional
 
-		return getFunctionality ( getRole ( R, "Role expression expected in isFunctional()" ) );
+		return getFunctionality(r);
 	}
 		/// @return true iff data role is functional
 	bool isFunctional ( const TDRoleExpr* R )
 	{
 		preprocessKB();	// ensure KB is ready to answer the query
-		if ( getExpressionManager()->isUniversalRole(R) )
+		TRole* r = getRole ( R, "Role expression expected in isFunctional()" );
+		if ( unlikely(r->isTop()) )
 			return false;	// universal role is not functional
-		if ( getExpressionManager()->isEmptyRole(R) )
+		if ( unlikely(r->isBottom()) )
 			return true;	// empty role is functional
 
-		return getFunctionality ( getRole ( R, "Role expression expected in isFunctional()" ) );
+		return getFunctionality(r);
 	}
 		/// @return true iff role is inverse-functional
 	bool isInverseFunctional ( const TORoleExpr* R )
 	{
 		preprocessKB();	// ensure KB is ready to answer the query
-		if ( getExpressionManager()->isUniversalRole(R) )
+		TRole* r = getRole ( R, "Role expression expected in isInverseFunctional()" )->inverse();
+		if ( unlikely(r->isTop()) )
 			return false;	// universal role is not functional
-		if ( getExpressionManager()->isEmptyRole(R) )
+		if ( unlikely(r->isBottom()) )
 			return true;	// empty role is functional
 
-		return getFunctionality ( getRole ( R, "Role expression expected in isInverseFunctional()" )->inverse() );
+		return getFunctionality(r);
 	}
 		/// @return true iff role is transitive
 	bool isTransitive ( const TORoleExpr* R )
 	{
 		preprocessKB();	// ensure KB is ready to answer the query
-		if ( getExpressionManager()->isUniversalRole(R) )
+		TRole* r = getRole ( R, "Role expression expected in isTransitive()" );
+		if ( unlikely(r->isTop()) )
 			return true;	// universal role is transitive
-		if ( getExpressionManager()->isEmptyRole(R) )
+		if ( unlikely(r->isBottom()) )
 			return true;	// empty role is transitive
 
-		TRole* r = getRole ( R, "Role expression expected in isTransitive()" );
 		if ( !r->isTransitivityKnown() )	// calculate transitivity
 			r->setTransitive(checkTransitivity(e(R)));
 		return r->isTransitive();
@@ -815,12 +872,12 @@ public:
 	bool isSymmetric ( const TORoleExpr* R )
 	{
 		preprocessKB();	// ensure KB is ready to answer the query
-		if ( getExpressionManager()->isUniversalRole(R) )
+		TRole* r = getRole ( R, "Role expression expected in isSymmetric()" );
+		if ( unlikely(r->isTop()) )
 			return true;	// universal role is symmetric
-		if ( getExpressionManager()->isEmptyRole(R) )
+		if ( unlikely(r->isBottom()) )
 			return true;	// empty role is symmetric
 
-		TRole* r = getRole ( R, "Role expression expected in isSymmetric()" );
 		if ( !r->isSymmetryKnown() )	// calculate symmetry
 			r->setSymmetric(checkSymmetry(e(R)));
 		return r->isSymmetric();
@@ -829,12 +886,12 @@ public:
 	bool isAsymmetric ( const TORoleExpr* R )
 	{
 		preprocessKB();	// ensure KB is ready to answer the query
-		if ( getExpressionManager()->isUniversalRole(R) )
+		TRole* r = getRole ( R, "Role expression expected in isAsymmetric()" );
+		if ( unlikely(r->isTop()) )
 			return false;	// universal role is not asymmetric
-		if ( getExpressionManager()->isEmptyRole(R) )
+		if ( unlikely(r->isBottom()) )
 			return true;	// empty role is asymmetric
 
-		TRole* r = getRole ( R, "Role expression expected in isAsymmetric()" );
 		if ( !r->isAsymmetryKnown() )	// calculate asymmetry
 			r->setAsymmetric(getTBox()->isDisjointRoles(r,r->inverse()));
 		return r->isAsymmetric();
@@ -843,12 +900,12 @@ public:
 	bool isReflexive ( const TORoleExpr* R )
 	{
 		preprocessKB();	// ensure KB is ready to answer the query
-		if ( getExpressionManager()->isUniversalRole(R) )
+		TRole* r = getRole ( R, "Role expression expected in isReflexive()" );
+		if ( unlikely(r->isTop()) )
 			return true;	// universal role is reflexive
-		if ( getExpressionManager()->isEmptyRole(R) )
+		if ( unlikely(r->isBottom()) )
 			return false;	// empty role is not reflexive
 
-		TRole* r = getRole ( R, "Role expression expected in isReflexive()" );
 		if ( !r->isReflexivityKnown() )	// calculate reflexivity
 			r->setReflexive(checkReflexivity(e(R)));
 		return r->isReflexive();
@@ -857,12 +914,12 @@ public:
 	bool isIrreflexive ( const TORoleExpr* R )
 	{
 		preprocessKB();	// ensure KB is ready to answer the query
-		if ( getExpressionManager()->isUniversalRole(R) )
+		TRole* r = getRole ( R, "Role expression expected in isIrreflexive()" );
+		if ( unlikely(r->isTop()) )
 			return false;	// universal role is not irreflexive
-		if ( getExpressionManager()->isEmptyRole(R) )
+		if ( unlikely(r->isBottom()) )
 			return true;	// empty role is irreflexive
 
-		TRole* r = getRole ( R, "Role expression expected in isIrreflexive()" );
 		if ( !r->isIrreflexivityKnown() )	// calculate irreflexivity
 			r->setIrreflexive(getTBox()->isIrreflexive(r));
 		return r->isIrreflexive();
@@ -871,61 +928,67 @@ public:
 	bool isSubRoles ( const TORoleExpr* R, const TORoleExpr* S )
 	{
 		preprocessKB();	// ensure KB is ready to answer the query
-		if ( getExpressionManager()->isEmptyRole(R) || getExpressionManager()->isUniversalRole(S) )
+		TRole* r = getRole ( R, "Role expression expected in isSubRoles()" );
+		TRole* s = getRole ( S, "Role expression expected in isSubRoles()" );
+		if ( unlikely(r->isBottom()) || unlikely(s->isTop()) )
 			return true;	// \bot [= X [= \top
-		if ( getExpressionManager()->isUniversalRole(R) && getExpressionManager()->isEmptyRole(S) )
+		if ( unlikely(r->isTop()) && unlikely(s->isBottom()) )
 			return false;	// as \top [= \bot leads to inconsistent ontology
 
 		// told case first
-		if ( *getRole ( R, "Role expression expected in isSubRoles()" ) <=
-			 *getRole ( S, "Role expression expected in isSubRoles()" ) )
-			return true;
+		if ( likely(!r->isTop() && !s->isBottom()) )
+		{
+			if ( *r <= *s )
+				return true;
+		}
 		// check the general case
 		// FIXME!! cache it later
-		DLTree* r = e(R), *s = e(S);
 		return checkRoleSubsumption ( r, s );
 	}
 		/// @return true if R is a sub-role of S
 	bool isSubRoles ( const TDRoleExpr* R, const TDRoleExpr* S )
 	{
 		preprocessKB();	// ensure KB is ready to answer the query
-		if ( getExpressionManager()->isEmptyRole(R) || getExpressionManager()->isUniversalRole(S) )
+		TRole* r = getRole ( R, "Role expression expected in isSubRoles()" );
+		TRole* s = getRole ( S, "Role expression expected in isSubRoles()" );
+		if ( unlikely(r->isBottom()) || unlikely(s->isTop()) )
 			return true;	// \bot [= X [= \top
-		if ( getExpressionManager()->isUniversalRole(R) && getExpressionManager()->isEmptyRole(S) )
+		if ( unlikely(r->isTop()) && unlikely(s->isBottom()) )
 			return false;	// as \top [= \bot leads to inconsistent ontology
 
 		// told case first
-		if ( *getRole ( R, "Role expression expected in isSubRoles()" ) <=
-			 *getRole ( S, "Role expression expected in isSubRoles()" ) )
-			return true;
+		if ( likely(!r->isTop() && !s->isBottom()) )
+		{
+			if ( *r <= *s )
+				return true;
+		}
 		// check the general case
 		// FIXME!! cache it later
-		DLTree* r = e(R), *s = e(S);
 		return checkRoleSubsumption ( r, s );
 	}
 		/// @return true iff two roles are disjoint
 	bool isDisjointRoles ( const TORoleExpr* R, const TORoleExpr* S )
 	{
 		preprocessKB();	// ensure KB is ready to answer the query
-		if ( getExpressionManager()->isUniversalRole(R) || getExpressionManager()->isUniversalRole(S) )
+		TRole* r = getRole ( R, "Role expression expected in isDisjointRoles()" );
+		TRole* s = getRole ( S, "Role expression expected in isDisjointRoles()" );
+		if ( unlikely(r->isTop()) || unlikely(s->isTop()) )
 			return false;	// universal role is not disjoint with anything
-		if ( getExpressionManager()->isEmptyRole(R) || getExpressionManager()->isEmptyRole(S) )
+		if ( unlikely(r->isBottom()) || unlikely(s->isBottom()) )
 			return true;	// empty role is disjoint with everything
-		return getTBox()->isDisjointRoles (
-			getRole ( R, "Role expression expected in isDisjointRoles()" ),
-			getRole ( S, "Role expression expected in isDisjointRoles()" ) );
+		return getTBox()->isDisjointRoles ( r, s );
 	}
 		/// @return true iff two roles are disjoint
 	bool isDisjointRoles ( const TDRoleExpr* R, const TDRoleExpr* S )
 	{
 		preprocessKB();	// ensure KB is ready to answer the query
-		if ( getExpressionManager()->isUniversalRole(R) || getExpressionManager()->isUniversalRole(S) )
+		TRole* r = getRole ( R, "Role expression expected in isDisjointRoles()" );
+		TRole* s = getRole ( S, "Role expression expected in isDisjointRoles()" );
+		if ( unlikely(r->isTop()) || unlikely(s->isTop()) )
 			return false;	// universal role is not disjoint with anything
-		if ( getExpressionManager()->isEmptyRole(R) || getExpressionManager()->isEmptyRole(S) )
+		if ( unlikely(r->isBottom()) || unlikely(s->isBottom()) )
 			return true;	// empty role is disjoint with everything
-		return getTBox()->isDisjointRoles (
-			getRole ( R, "Role expression expected in isDisjointRoles()" ),
-			getRole ( S, "Role expression expected in isDisjointRoles()" ) );
+		return getTBox()->isDisjointRoles ( r, s );
 	}
 		/// @return true iff all the roles in a arg-list are pairwise disjoint
 	bool isDisjointRoles ( void );
@@ -933,11 +996,10 @@ public:
 	bool isSubChain ( const TORoleExpr* R )
 	{
 		preprocessKB();	// ensure KB is ready to answer the query
-		if ( getExpressionManager()->isUniversalRole(R) )
+		TRole* r = getRole ( R, "Role expression expected in isSubChain()" );
+		if ( unlikely(r->isTop()) )
 			return true;	// universal role is a super of any chain
-		if ( getExpressionManager()->isEmptyRole(R) )	// FIXME!! true in case empty role is in chain
-			return false;	// empty role is not a super of any chain
-		return checkSubChain ( getRole ( R, "Role expression expected in isSubChain()" ) );
+		return checkSubChain ( getExpressionManager()->getArgList(), r );
 	}
 
 	// single satisfiability
@@ -961,8 +1023,8 @@ public:
 		preprocessKB();
 		if ( isNameOrConst(D) && likely(isNameOrConst(C)) )
 			return checkSub ( getTBox()->getCI(TreeDeleter(e(C))), getTBox()->getCI(TreeDeleter(e(D))) );
-
-		return !checkSat ( getExpressionManager()->And ( C, getExpressionManager()->Not(D) ) );
+		DLTree* nD = createSNFNot(e(D));
+		return !checkSatTree ( createSNFAnd (e(C), nD) );
 	}
 		/// @return true iff C is disjoint with D; that is, (C and D) is unsatisfiable
 	bool isDisjoint ( const TConceptExpr* C, const TConceptExpr* D ) { return !isSatisfiable(getExpressionManager()->And(C,D)); }
@@ -1115,7 +1177,7 @@ public:
 		// if not, just go 1 level down and apply the actor regardless of what's found
 		// FIXME!! check again after bucket-method will be implemented
 		for ( TaxonomyVertex::iterator p = cachedVertex->begin(/*upDirection=*/false),
-				p_end = cachedVertex->end(/*upDirection=*/false); p < p_end; ++p )
+				p_end = cachedVertex->end(/*upDirection=*/false); p != p_end; ++p )
 			actor.apply(**p);
 	}
 
@@ -1180,16 +1242,43 @@ public:
 	{
 		preprocessKB();
 		setUpCache ( C, csSat );
-		return getTBox()->buildCompletionTree(cachedConcept);
+		const TCGNode* ret = getTBox()->buildCompletionTree(cachedConcept);
+		if ( KE == NULL )	// init KB after the sat test to reduce the number of DAG adjustments
+			KE = new KnowledgeExplorer ( getTBox(), getExpressionManager() );
+		return ret;
 	}
 		/// build the set of data neighbours of a NODE, put the set of data roles into the RESULT variable
-	void getDataRoles ( const TCGNode* node, TCGRoleSet& Result, bool onlyDet );
+	void getDataRoles ( const TCGNode* node, TCGRoleSet& Result, bool onlyDet ) { Result = KE->getDataRoles ( node, onlyDet ); }
 		/// build the set of object neighbours of a NODE, put the set of object roles and inverses into the RESULT variable
-	void getObjectRoles ( const TCGNode* node, TCGRoleSet& Result, bool onlyDet, bool needIncoming );
+	void getObjectRoles ( const TCGNode* node, TCGRoleSet& Result, bool onlyDet, bool needIncoming ) { Result = KE->getObjectRoles ( node, onlyDet, needIncoming ); }
 		/// build the set of neighbours of a NODE via role ROLE; put the resulting list into RESULT
-	void getNeighbours ( const TCGNode* node, TRoleExpr* role, TCGNodeVec& Result );
+	void getNeighbours ( const TCGNode* node, TRoleExpr* role, TCGNodeVec& Result )
+		{ Result = KE->getNeighbours ( node, getRole ( role, "Role expression expected in getNeighbours() method" ) ); }
 		/// put into RESULT all the expressions from the NODE label; if ONLYDET is true, return only deterministic elements
-	void getLabel ( const TCGNode* node, TCGItemVec& Result, bool onlyDet );
+	void getLabel ( const TCGNode* node, TCGItemVec& Result, bool onlyDet ) { Result = KE->getLabel ( node, onlyDet ); }
+
+	//----------------------------------------------------------------------------------
+	// atomic decomposition queries
+	//----------------------------------------------------------------------------------
+
+		/// create new atomic decomposition of the loaded ontology using TYPE. @return size of the AD
+	unsigned int getAtomicDecompositionSize ( bool useSemantic, ModuleType moduleType );
+		/// get a set of axioms that corresponds to the atom with the id INDEX
+	const TOntologyAtom::AxiomSet& getAtomAxioms ( unsigned int index ) const;
+		/// get a set of axioms that corresponds to the module of the atom with the id INDEX
+	const TOntologyAtom::AxiomSet& getAtomModule ( unsigned int index ) const;
+		/// get a set of atoms on which atom with index INDEX depends
+	const TOntologyAtom::AtomSet& getAtomDependents ( unsigned int index ) const;
+
+	//----------------------------------------------------------------------------------
+	// modularity queries
+	//----------------------------------------------------------------------------------
+
+		/// get a set of axioms that corresponds to the atom with the id INDEX
+	const AxiomVec& getModule ( bool useSemantic, ModuleType moduleType );
+		/// get a set of axioms that corresponds to the atom with the id INDEX
+	const AxiomVec& getNonLocal ( bool useSemantic, ModuleType moduleType );
+
 }; // ReasoningKernel
 
 #endif
