@@ -1,5 +1,5 @@
 /* This file is part of the FaCT++ DL reasoner
-Copyright (C) 2003-2011 by Dmitry Tsarkov
+Copyright (C) 2003-2012 by Dmitry Tsarkov
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -16,10 +16,11 @@ License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
+#include <iomanip>
+
 #include "Reasoner.h"
 #include "logging.h"
 #include "eFPPTimeout.h"
-#include "Modularity.h"
 
 // comment the line out for printing tree info before save and after restoring
 //#define __DEBUG_SAVE_RESTORE
@@ -37,7 +38,7 @@ DlSatTester :: DlSatTester ( TBox& tbox )
 	, CGraph(1,this)
 	, TODO(tBox.PriorityMatrix)
 	, DTReasoner(tbox.DLHeap)
-	// It's unsafe to have a cache that touchs a nominal in a node; set flagNominals to prevent it
+	// It's unsafe to have a cache that touches a nominal in a node; set flagNominals to prevent it
 	, newNodeCache ( true, tBox.nC, tBox.nR )
 	, newNodeEdges ( false, tBox.nC, tBox.nR )
 	, GCIs(tbox.GCIs)
@@ -56,7 +57,7 @@ DlSatTester :: DlSatTester ( TBox& tbox )
 	// init blocking statistics
 	clearBlockingStat();
 
-	resetSessionFlags ();
+	resetSessionFlags();
 }
 
 /// prerpare Nominal Reasoner to a new job
@@ -496,7 +497,7 @@ DlSatTester :: performAfterReasoning ( void )
 	if ( LLM.isWritable(llGTA) )
 	{
 		logIndentation();
-		LL << "[*ub:";
+		LL << "ub:";
 	}
 	CGraph.retestCGBlockedStatus();
 	if ( LLM.isWritable(llGTA) )
@@ -510,7 +511,7 @@ DlSatTester :: performAfterReasoning ( void )
 		if ( LLM.isWritable(llGTA) )
 		{
 			logIndentation();
-			LL << "[*split:";
+			LL << "split:";
 		}
 		bool clash = checkSplitRules();
 		if ( LLM.isWritable(llGTA) )
@@ -642,7 +643,7 @@ void DlSatTester :: restore ( unsigned int newTryLevel )
 	fpp_assert ( !Stack.empty () );
 	fpp_assert ( newTryLevel > 0 );
 
-	// skip all intermediate restorings
+	// skip all intermediate restores
 	setCurLevel(newTryLevel);
 
 	// update split level
@@ -675,20 +676,22 @@ void DlSatTester :: restore ( unsigned int newTryLevel )
 void DlSatTester :: logIndentation ( void ) const
 {
 	CHECK_LL_RETURN(llGTA);	// useless but safe
-	LL << "\n";
-	for ( register unsigned int i = getCurLevel(); --i; )
-		LL << " ";
+	LL << "\n" << std::setw(getCurLevel()) << "[";
 }
 void DlSatTester :: logStartEntry ( void ) const
 {
 	CHECK_LL_RETURN(llGTA);	// useless but safe
 	logIndentation();
-	LL << "[*(";
+	LL << "(";
 	curNode->logNode ();
 	LL << "," << curConcept << "){";
 	if ( isNegative (curConcept.bp()) )
 		LL << "~";
-	LL << DLHeap[curConcept].getTagName() << "}:";
+	const DLVertex& v = DLHeap[curConcept];
+	LL << v.getTagName();
+	if ( const TNamedEntry* entry = v.getConcept() )
+		LL << "(" << entry->getName() << ")";
+	LL << "}:";
 }
 
 void DlSatTester :: logFinishEntry ( bool res ) const
