@@ -1,5 +1,5 @@
 /* This file is part of the FaCT++ DL reasoner
-Copyright (C) 2011 by Dmitry Tsarkov
+Copyright (C) 2011-2013 by Dmitry Tsarkov
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -19,40 +19,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 #ifndef SYNLOCCHECKER_H
 #define SYNLOCCHECKER_H
 
-#include "tSignature.h"
+#include "LocalityChecker.h"
 
 // forward declarations
 class BotEquivalenceEvaluator;
 class TopEquivalenceEvaluator;
 
-/// helper class to set signature and locality class
-class SigAccessor
-{
-protected:	// members
-		/// signature of a module
-	const TSignature* sig;
-
-public:		// interface
-		/// init c'tor
-	SigAccessor ( const TSignature* s ) : sig(s) {}
-		/// empty d'tor
-	virtual ~SigAccessor ( void ) {}
-
-		/// @return true iff concepts are treated as TOPs
-	bool topCLocal ( void ) const { return sig->topCLocal(); }
-		/// @return true iff roles are treated as TOPs
-	bool topRLocal ( void ) const { return sig->topRLocal(); }
-		/// @return true iff SIGnature does NOT contain given expression
-	bool nc ( const TNamedEntity* expr ) const { return !sig->contains(expr); }
-
-		/// @return true iff EXPR is a top datatype
-	static bool isTopDT ( const TDLExpression* expr ) { return dynamic_cast<const TDLDataTop*>(expr) != NULL; }
-		/// @return true iff EXPR is a top datatype or a built-in datatype; FIXME for now -- just top
-	static bool isTopOrBuiltInDT ( const TDLExpression* expr ) { return isTopDT(expr); }
-		/// @return true iff EXPR is a top datatype or an infinite built-in datatype; FIXME for now -- just top
-	static bool isTopOrBuiltInInfDT ( const TDLExpression* expr ) { return isTopDT(expr); }
-}; // SigAccessor
-
 /// check whether class expressions are equivalent to bottom wrt given locality class
 class BotEquivalenceEvaluator: protected SigAccessor, public DLExpressionVisitorEmpty
 {
@@ -65,14 +37,53 @@ protected:	// members
 protected:	// methods
 		/// check whether the expression is top-equivalent
 	bool isTopEquivalent ( const TDLExpression& expr );
-		/// convinience helper
+		/// convenience helper
 	bool isTopEquivalent ( const TDLExpression* expr ) { return isTopEquivalent(*expr); }
-		/// @return true iff role expression in equivalent to const wrt locality
-	bool isREquivalent ( const TDLExpression* expr ) { return topRLocal() ? isTopEquivalent(expr) : isBotEquivalent(*expr); }
+
+	// non-empty Concept/Data expression
+
+		/// @return true iff C^I is non-empty
+	bool isBotDistinct ( const TDLExpression* C )
+	{
+		// TOP is non-empty
+		if ( isTopEquivalent(C) )
+			return true;
+		// built-in DT are non-empty
+		if ( dynamic_cast<const TDLDataTypeName*>(C) )
+			return true;
+		// FIXME!! that's it for now
+		return false;
+	}
+
+	// cardinality of a concept/data expression interpretation
+
+		/// @return true if #C^I > n
+	bool isCardLargerThan ( const TDLExpression* C, unsigned int n )
+	{
+		if ( n == 0 )	// non-empty is enough
+			return isBotDistinct(C);
+		if ( const TDLDataTypeName* namedDT = dynamic_cast<const TDLDataTypeName*>(C) )
+		{	// string/time are infinite DT
+			std::string name = namedDT->getName();
+			if ( name == TDataTypeManager::getStrTypeName() || name == TDataTypeManager::getTimeTypeName() )
+				return true;
+		}
+		// FIXME!! try to be more precise
+		return false;
+	}
+
+	// QCRs
+
+		/// @return true iff (>= n R.C) is botEq
+	bool isMinBotEquivalent ( unsigned int n, const TDLRoleExpression* R, const TDLExpression* C )
+		{ return (n > 0) && (isBotEquivalent(R) || isBotEquivalent(C)); }
+		/// @return true iff (<= n R.C) is botEq
+	bool isMaxBotEquivalent ( unsigned int n, const TDLRoleExpression* R, const TDLExpression* C )
+		{ return isTopEquivalent(R) && isCardLargerThan ( C, n ); }
 
 public:		// interface
 		/// init c'tor
-	BotEquivalenceEvaluator ( const TSignature* s ) : SigAccessor(s) {}
+	BotEquivalenceEvaluator ( const TSignature* s ) : SigAccessor(s), isBotEq(false) {}
 		/// empty d'tor
 	virtual ~BotEquivalenceEvaluator ( void ) {}
 
@@ -91,8 +102,8 @@ public:		// interface
 
 public:		// visitor interface
 	// concept expressions
-	virtual void visit ( const TDLConceptTop& expr ATTR_UNUSED ) { isBotEq = false; }
-	virtual void visit ( const TDLConceptBottom& expr ATTR_UNUSED ) { isBotEq = true; }
+	virtual void visit ( const TDLConceptTop& ) { isBotEq = false; }
+	virtual void visit ( const TDLConceptBottom& ) { isBotEq = true; }
 	virtual void visit ( const TDLConceptName& expr ) { isBotEq = !topCLocal() && nc(expr.getEntity()); }
 	virtual void visit ( const TDLConceptNot& expr ) { isBotEq = isTopEquivalent(expr.getC()); }
 	virtual void visit ( const TDLConceptAnd& expr )
@@ -104,67 +115,94 @@ public:		// visitor interface
 	}
 	virtual void visit ( const TDLConceptOr& expr )
 	{
-		for ( TDLConceptAnd::iterator p = expr.begin(), p_end = expr.end(); p != p_end; ++p )
+		for ( TDLConceptOr::iterator p = expr.begin(), p_end = expr.end(); p != p_end; ++p )
 			if ( !isBotEquivalent(*p) )	// here isBotEq is false, so just return
 				return;
 		isBotEq = true;
 	}
 	virtual void visit ( const TDLConceptOneOf& expr ) { isBotEq = expr.empty(); }
-	virtual void visit ( const TDLConceptObjectSelf& expr ) { isBotEq = !topRLocal() && isBotEquivalent(expr.getOR()); }
-	virtual void visit ( const TDLConceptObjectValue& expr ) { isBotEq = !topRLocal() && isBotEquivalent(expr.getOR()); }
+	virtual void visit ( const TDLConceptObjectSelf& expr ) { isBotEq = isBotEquivalent(expr.getOR()); }
+	virtual void visit ( const TDLConceptObjectValue& expr ) { isBotEq = isBotEquivalent(expr.getOR()); }
 	virtual void visit ( const TDLConceptObjectExists& expr )
-	{
-		isBotEq = isBotEquivalent(expr.getC());
-		if ( !topRLocal() )
-			isBotEq |= isBotEquivalent(expr.getOR());
-	}
+		{ isBotEq = isMinBotEquivalent ( 1, expr.getOR(), expr.getC() ); }
 	virtual void visit ( const TDLConceptObjectForall& expr )
-		{ isBotEq = topRLocal() && isTopEquivalent(expr.getOR()) && isBotEquivalent(expr.getC()); }
+		{ isBotEq = isTopEquivalent(expr.getOR()) && isBotEquivalent(expr.getC()); }
 	virtual void visit ( const TDLConceptObjectMinCardinality& expr )
-		{ isBotEq = expr.getNumber() > 0 && (isBotEquivalent(expr.getC()) || (!topRLocal() && isBotEquivalent(expr.getOR())) ); }
+		{ isBotEq = isMinBotEquivalent ( expr.getNumber(), expr.getOR(), expr.getC() ); }
 	virtual void visit ( const TDLConceptObjectMaxCardinality& expr )
-		{ isBotEq = topRLocal() && (expr.getNumber() > 0) && isTopEquivalent(expr.getOR()) && isTopEquivalent(expr.getC()); }
+		{ isBotEq = isMaxBotEquivalent ( expr.getNumber(), expr.getOR(), expr.getC() ); }
 	virtual void visit ( const TDLConceptObjectExactCardinality& expr )
 	{
-		isBotEq = expr.getNumber() > 0 &&
-			( isBotEquivalent(expr.getC()) || ( isREquivalent(expr.getOR()) && (topRLocal() ? isTopEquivalent(expr.getC()) : true) ) );
+		unsigned int n = expr.getNumber();
+		const TDLObjectRoleExpression* R = expr.getOR();
+		const TDLConceptExpression* C = expr.getC();
+		isBotEq = isMinBotEquivalent ( n, R, C ) || isMaxBotEquivalent ( n, R, C );
 	}
-	virtual void visit ( const TDLConceptDataValue& expr ) { isBotEq = !topRLocal() && isBotEquivalent(expr.getDR()); }
-	virtual void visit ( const TDLConceptDataExists& expr ) { isBotEq = !topRLocal() && isBotEquivalent(expr.getDR()); }
-	virtual void visit ( const TDLConceptDataForall& expr ) { isBotEq = topRLocal() && isTopEquivalent(expr.getDR()) && !isTopDT(expr.getExpr()); }
+	virtual void visit ( const TDLConceptDataValue& expr )
+		{ isBotEq = isBotEquivalent(expr.getDR()); }
+	virtual void visit ( const TDLConceptDataExists& expr )
+		{ isBotEq = isMinBotEquivalent ( 1, expr.getDR(), expr.getExpr() ); }
+	virtual void visit ( const TDLConceptDataForall& expr )
+		{ isBotEq = isTopEquivalent(expr.getDR()) && !isTopEquivalent(expr.getExpr()); }
 	virtual void visit ( const TDLConceptDataMinCardinality& expr )
-		{ isBotEq = !topRLocal() && (expr.getNumber() > 0) && isBotEquivalent(expr.getDR()); }
+		{ isBotEq = isMinBotEquivalent ( expr.getNumber(), expr.getDR(), expr.getExpr() ); }
 	virtual void visit ( const TDLConceptDataMaxCardinality& expr )
-	{
-		isBotEq = topRLocal() && isTopEquivalent(expr.getDR()) &&
-			( expr.getNumber() <= 1 ? isTopOrBuiltInDT(expr.getExpr()) : isTopOrBuiltInInfDT(expr.getExpr()) );
-	}
+		{ isBotEq = isMaxBotEquivalent ( expr.getNumber(), expr.getDR(), expr.getExpr() ); }
 	virtual void visit ( const TDLConceptDataExactCardinality& expr )
 	{
-		isBotEq = isREquivalent(expr.getDR()) && ( topRLocal()
-			? ( expr.getNumber() == 0
-				? isTopOrBuiltInDT(expr.getExpr())
-				: isTopOrBuiltInInfDT(expr.getExpr()) )
-			: expr.getNumber() > 0 );
+		unsigned int n = expr.getNumber();
+		const TDLDataRoleExpression* R = expr.getDR();
+		const TDLDataExpression* D = expr.getExpr();
+		isBotEq = isMinBotEquivalent ( n, R, D ) || isMaxBotEquivalent ( n, R, D );
 	}
 
 	// object role expressions
-	virtual void visit ( const TDLObjectRoleTop& expr ATTR_UNUSED ) { isBotEq = false; }
-	virtual void visit ( const TDLObjectRoleBottom& expr ATTR_UNUSED ) { isBotEq = true; }
+	virtual void visit ( const TDLObjectRoleTop& ) { isBotEq = false; }
+	virtual void visit ( const TDLObjectRoleBottom& ) { isBotEq = true; }
 	virtual void visit ( const TDLObjectRoleName& expr ) { isBotEq = !topRLocal() && nc(expr.getEntity()); }
 	virtual void visit ( const TDLObjectRoleInverse& expr ) { isBotEq = isBotEquivalent(expr.getOR()); }
 	virtual void visit ( const TDLObjectRoleChain& expr )
 	{
+		isBotEq = true;
 		for ( TDLObjectRoleChain::iterator p = expr.begin(), p_end = expr.end(); p != p_end; ++p )
 			if ( isBotEquivalent(*p) )	// isBotEq is true here
 				return;
 		isBotEq = false;
 	}
+		// equivalent to R(x,y) and C(x), so copy behaviour from ER.X
+	virtual void visit ( const TDLObjectRoleProjectionFrom& expr )
+		{ isBotEq = isMinBotEquivalent ( 1, expr.getOR(), expr.getC() ); }
+		// equivalent to R(x,y) and C(y), so copy behaviour from ER.X
+	virtual void visit ( const TDLObjectRoleProjectionInto& expr )
+		{ isBotEq = isMinBotEquivalent ( 1, expr.getOR(), expr.getC() ); }
 
 	// data role expressions
-	virtual void visit ( const TDLDataRoleTop& expr ATTR_UNUSED ) { isBotEq = false; }
-	virtual void visit ( const TDLDataRoleBottom& expr ATTR_UNUSED ) { isBotEq = true; }
+	virtual void visit ( const TDLDataRoleTop& ) { isBotEq = false; }
+	virtual void visit ( const TDLDataRoleBottom& ) { isBotEq = true; }
 	virtual void visit ( const TDLDataRoleName& expr ) { isBotEq = !topRLocal() && nc(expr.getEntity()); }
+
+	// data expressions
+	virtual void visit ( const TDLDataTop& ) { isBotEq = false; }
+	virtual void visit ( const TDLDataBottom& ) { isBotEq = true; }
+	virtual void visit ( const TDLDataTypeName& ) { isBotEq = false; }
+	virtual void visit ( const TDLDataTypeRestriction& ) { isBotEq = false; }
+	virtual void visit ( const TDLDataValue& ) { isBotEq = false; }
+	virtual void visit ( const TDLDataNot& expr ) { isBotEq = isTopEquivalent(expr.getExpr()); }
+	virtual void visit ( const TDLDataAnd& expr )
+	{
+		for ( TDLDataAnd::iterator p = expr.begin(), p_end = expr.end(); p != p_end; ++p )
+			if ( isBotEquivalent(*p) )	// here isBotEq is true, so just return
+				return;
+		isBotEq = false;
+	}
+	virtual void visit ( const TDLDataOr& expr )
+	{
+		for ( TDLDataOr::iterator p = expr.begin(), p_end = expr.end(); p != p_end; ++p )
+			if ( !isBotEquivalent(*p) )	// here isBotEq is false, so just return
+				return;
+		isBotEq = true;
+	}
+	virtual void visit ( const TDLDataOneOf& expr ) { isBotEq = expr.empty(); }
 }; // BotEquivalenceEvaluator
 
 /// check whether class expressions are equivalent to top wrt given locality class
@@ -179,14 +217,55 @@ protected:	// members
 protected:	// methods
 		/// check whether the expression is top-equivalent
 	bool isBotEquivalent ( const TDLExpression& expr ) { return BotEval->isBotEquivalent(expr); }
-		/// convinience helper
+		/// convenience helper
 	bool isBotEquivalent ( const TDLExpression* expr ) { return isBotEquivalent(*expr); }
-		/// @return true iff role expression in equivalent to const wrt locality
-	bool isREquivalent ( const TDLExpression* expr ) { return topRLocal() ? isTopEquivalent(*expr) : isBotEquivalent(*expr); }
+
+	// non-empty Concept/Data expression
+
+		/// @return true iff C^I is non-empty
+	bool isBotDistinct ( const TDLExpression* C )
+	{
+		// TOP is non-empty
+		if ( isTopEquivalent(C) )
+			return true;
+		// built-in DT are non-empty
+		if ( dynamic_cast<const TDLDataTypeName*>(C) )
+			return true;
+		// FIXME!! that's it for now
+		return false;
+	}
+
+	// cardinality of a concept/data expression interpretation
+
+		/// @return true if #C^I > n
+	bool isCardLargerThan ( const TDLExpression* C, unsigned int n )
+	{
+		if ( n == 0 )	// non-empty is enough
+			return isBotDistinct(C);
+		if ( dynamic_cast<const TDLDataExpression*>(C) && isTopEquivalent(C) )
+			return true;
+		if ( const TDLDataTypeName* namedDT = dynamic_cast<const TDLDataTypeName*>(C) )
+		{	// string/time are infinite DT
+			std::string name = namedDT->getName();
+			if ( name == TDataTypeManager::getStrTypeName() || name == TDataTypeManager::getTimeTypeName() )
+				return true;
+		}
+		// FIXME!! try to be more precise
+		return false;
+	}
+
+	// QCRs
+
+		/// @return true iff (>= n R.C) is topEq
+	bool isMinTopEquivalent ( unsigned int n, const TDLRoleExpression* R, const TDLExpression* C )
+		{ return (n == 0) || ( isTopEquivalent(R) && isCardLargerThan ( C, n-1 ) ); }
+		/// @return true iff (<= n R.C) is topEq
+	bool isMaxTopEquivalent ( unsigned int n ATTR_UNUSED, const TDLRoleExpression* R, const TDLExpression* C )
+		{ return isBotEquivalent(R) || isBotEquivalent(C); }
 
 public:		// interface
 		/// init c'tor
-	TopEquivalenceEvaluator ( const TSignature* s ) : SigAccessor(s) {}
+	TopEquivalenceEvaluator ( const TSignature* s ) : SigAccessor(s), isTopEq(false) {}
 		/// empty d'tor
 	virtual ~TopEquivalenceEvaluator ( void ) {}
 
@@ -205,8 +284,8 @@ public:		// interface
 
 public:		// visitor interface
 	// concept expressions
-	virtual void visit ( const TDLConceptTop& expr ATTR_UNUSED ) { isTopEq = true; }
-	virtual void visit ( const TDLConceptBottom& expr ATTR_UNUSED ) { isTopEq = false; }
+	virtual void visit ( const TDLConceptTop& ) { isTopEq = true; }
+	virtual void visit ( const TDLConceptBottom& ) { isTopEq = false; }
 	virtual void visit ( const TDLConceptName& expr ) { isTopEq = topCLocal() && nc(expr.getEntity()); }
 	virtual void visit ( const TDLConceptNot& expr ) { isTopEq = isBotEquivalent(expr.getC()); }
 	virtual void visit ( const TDLConceptAnd& expr )
@@ -223,53 +302,88 @@ public:		// visitor interface
 				return;
 		isTopEq = false;
 	}
-	virtual void visit ( const TDLConceptOneOf& expr ATTR_UNUSED ) { isTopEq = false; }
-	virtual void visit ( const TDLConceptObjectSelf& expr ) { isTopEq = topRLocal() && isTopEquivalent(expr.getOR()); }
-	virtual void visit ( const TDLConceptObjectValue& expr ) { isTopEq = topRLocal() && isTopEquivalent(expr.getOR()); }
+	virtual void visit ( const TDLConceptOneOf& ) { isTopEq = false; }
+	virtual void visit ( const TDLConceptObjectSelf& expr ) { isTopEq = isTopEquivalent(expr.getOR()); }
+	virtual void visit ( const TDLConceptObjectValue& expr ) { isTopEq = isTopEquivalent(expr.getOR()); }
 	virtual void visit ( const TDLConceptObjectExists& expr )
-		{ isTopEq = topRLocal() && isTopEquivalent(expr.getOR()) && isTopEquivalent(expr.getC()); }
+		{ isTopEq = isMinTopEquivalent ( 1, expr.getOR(), expr.getC() ); }
 	virtual void visit ( const TDLConceptObjectForall& expr )
-		{ isTopEq = isTopEquivalent(expr.getC()) || (!topRLocal() && isBotEquivalent(expr.getOR())); }
+		{ isTopEq = isTopEquivalent(expr.getC()) || isBotEquivalent(expr.getOR()); }
 	virtual void visit ( const TDLConceptObjectMinCardinality& expr )
-		{ isTopEq = expr.getNumber() == 0 || (topRLocal() && isTopEquivalent(expr.getOR()) && isTopEquivalent(expr.getC())); }
+		{ isTopEq = isMinTopEquivalent ( expr.getNumber(), expr.getOR(), expr.getC() ); }
 	virtual void visit ( const TDLConceptObjectMaxCardinality& expr )
-		{ isTopEq = isBotEquivalent(expr.getC()) || (!topRLocal() && isBotEquivalent(expr.getOR())); }
+		{ isTopEq = isMaxTopEquivalent ( expr.getNumber(), expr.getOR(), expr.getC() ); }
 	virtual void visit ( const TDLConceptObjectExactCardinality& expr )
 	{
-		isTopEq = expr.getNumber() == 0 &&
-			( isBotEquivalent(expr.getC()) || (!topRLocal() && isBotEquivalent(expr.getOR())) );
+		unsigned int n = expr.getNumber();
+		const TDLObjectRoleExpression* R = expr.getOR();
+		const TDLConceptExpression* C = expr.getC();
+		isTopEq = isMinTopEquivalent ( n, R, C ) && isMaxTopEquivalent ( n, R, C );
 	}
-	virtual void visit ( const TDLConceptDataValue& expr ) { isTopEq = topRLocal() && isTopEquivalent(expr.getDR()); }
-	virtual void visit ( const TDLConceptDataExists& expr ) { isTopEq = topRLocal() && isTopEquivalent(expr.getDR()) && isTopOrBuiltInDT(expr.getExpr()); }
-	virtual void visit ( const TDLConceptDataForall& expr ) { isTopEq = isTopDT(expr.getExpr()) || (!topRLocal() && isBotEquivalent(expr.getDR())); }
+	virtual void visit ( const TDLConceptDataValue& expr )
+		{ isTopEq = isTopEquivalent(expr.getDR()); }
+	virtual void visit ( const TDLConceptDataExists& expr )
+		{ isTopEq = isMinTopEquivalent ( 1, expr.getDR(), expr.getExpr() ); }
+	virtual void visit ( const TDLConceptDataForall& expr ) { isTopEq = isTopEquivalent(expr.getExpr()) || isBotEquivalent(expr.getDR()); }
 	virtual void visit ( const TDLConceptDataMinCardinality& expr )
-	{
-		isTopEq = expr.getNumber() == 0;
-		if ( topRLocal() )
-			isTopEq |= isTopEquivalent(expr.getDR()) && ( expr.getNumber() == 1 ? isTopOrBuiltInDT(expr.getExpr()) : isTopOrBuiltInInfDT(expr.getExpr()) );
-	}
+		{ isTopEq = isMinTopEquivalent ( expr.getNumber(), expr.getDR(), expr.getExpr() ); }
 	virtual void visit ( const TDLConceptDataMaxCardinality& expr )
-		{ isTopEq = !topRLocal() && isBotEquivalent(expr.getDR()); }
+		{ isTopEq = isMaxTopEquivalent ( expr.getNumber(), expr.getDR(), expr.getExpr() ); }
 	virtual void visit ( const TDLConceptDataExactCardinality& expr )
-		{ isTopEq = !topRLocal() && (expr.getNumber() == 0) && isBotEquivalent(expr.getDR()); }
+	{
+		unsigned int n = expr.getNumber();
+		const TDLDataRoleExpression* R = expr.getDR();
+		const TDLDataExpression* D = expr.getExpr();
+		isTopEq = isMinTopEquivalent ( n, R, D ) && isMaxTopEquivalent ( n, R, D );
+	}
 
 	// object role expressions
-	virtual void visit ( const TDLObjectRoleTop& expr ATTR_UNUSED ) { isTopEq = true; }
-	virtual void visit ( const TDLObjectRoleBottom& expr ATTR_UNUSED ) { isTopEq = false; }
+	virtual void visit ( const TDLObjectRoleTop& ) { isTopEq = true; }
+	virtual void visit ( const TDLObjectRoleBottom& ) { isTopEq = false; }
 	virtual void visit ( const TDLObjectRoleName& expr ) { isTopEq = topRLocal() && nc(expr.getEntity()); }
 	virtual void visit ( const TDLObjectRoleInverse& expr ) { isTopEq = isTopEquivalent(expr.getOR()); }
 	virtual void visit ( const TDLObjectRoleChain& expr )
 	{
+		isTopEq = false;
 		for ( TDLObjectRoleChain::iterator p = expr.begin(), p_end = expr.end(); p != p_end; ++p )
 			if ( !isTopEquivalent(*p) )	// isTopEq is false here
 				return;
 		isTopEq = true;
 	}
+		// equivalent to R(x,y) and C(x), so copy behaviour from ER.X
+	virtual void visit ( const TDLObjectRoleProjectionFrom& expr )
+		{ isTopEq = isMinTopEquivalent ( 1, expr.getOR(), expr.getC() ); }
+		// equivalent to R(x,y) and C(y), so copy behaviour from ER.X
+	virtual void visit ( const TDLObjectRoleProjectionInto& expr )
+		{ isTopEq = isMinTopEquivalent ( 1, expr.getOR(), expr.getC() ); }
 
 	// data role expressions
-	virtual void visit ( const TDLDataRoleTop& expr ATTR_UNUSED ) { isTopEq = true; }
-	virtual void visit ( const TDLDataRoleBottom& expr ATTR_UNUSED ) { isTopEq = false; }
+	virtual void visit ( const TDLDataRoleTop& ) { isTopEq = true; }
+	virtual void visit ( const TDLDataRoleBottom& ) { isTopEq = false; }
 	virtual void visit ( const TDLDataRoleName& expr ) { isTopEq = topRLocal() && nc(expr.getEntity()); }
+
+	// data expressions
+	virtual void visit ( const TDLDataTop& ) { isTopEq = true; }
+	virtual void visit ( const TDLDataBottom& ) { isTopEq = false; }
+	virtual void visit ( const TDLDataTypeName& ) { isTopEq = false; }
+	virtual void visit ( const TDLDataTypeRestriction& ) { isTopEq = false; }
+	virtual void visit ( const TDLDataValue& ) { isTopEq = false; }
+	virtual void visit ( const TDLDataNot& expr ) { isTopEq = isBotEquivalent(expr.getExpr()); }
+	virtual void visit ( const TDLDataAnd& expr )
+	{
+		for ( TDLDataAnd::iterator p = expr.begin(), p_end = expr.end(); p != p_end; ++p )
+			if ( !isTopEquivalent(*p) )	// here isTopEq is false, so just return
+				return;
+		isTopEq = true;
+	}
+	virtual void visit ( const TDLDataOr& expr )
+	{
+		for ( TDLDataOr::iterator p = expr.begin(), p_end = expr.end(); p != p_end; ++p )
+			if ( isTopEquivalent(*p) )	// here isTopEq is true, so just return
+				return;
+		isTopEq = false;
+	}
+	virtual void visit ( const TDLDataOneOf& ) { isTopEq = false; }
 }; // TopEquivalenceEvaluator
 
 inline bool
@@ -280,15 +394,13 @@ BotEquivalenceEvaluator :: isTopEquivalent ( const TDLExpression& expr )
 
 
 /// syntactic locality checker for DL axioms
-class SyntacticLocalityChecker: protected SigAccessor, public DLAxiomVisitor
+class SyntacticLocalityChecker: public LocalityChecker
 {
 protected:	// members
 		/// top evaluator
 	TopEquivalenceEvaluator TopEval;
 		/// bottom evaluator
 	BotEquivalenceEvaluator BotEval;
-		/// remember the axiom locality value here
-	bool isLocal;
 
 protected:	// methods
 		/// @return true iff EXPR is top equivalent
@@ -301,7 +413,7 @@ protected:	// methods
 public:		// interface
 		/// init c'tor
 	SyntacticLocalityChecker ( const TSignature* s )
-		: SigAccessor(s)
+		: LocalityChecker(s)
 		, TopEval(s)
 		, BotEval(s)
 	{
@@ -311,26 +423,8 @@ public:		// interface
 		/// empty d'tor
 	virtual ~SyntacticLocalityChecker ( void ) {}
 
-	// set fields
-
-		/// @return true iff an AXIOM is local wrt defined policy
-	bool local ( const TDLAxiom* axiom )
-	{
-		axiom->accept(*this);
-		return isLocal;
-	}
-		/// load ontology to a given KB
-	virtual void visitOntology ( TOntology& ontology )
-	{
-		for ( TOntology::iterator p = ontology.begin(), p_end = ontology.end(); p < p_end; ++p )
-			if ( (*p)->isUsed() )
-			{
-				(*p)->accept(*this);
-			}
-	}
-
 public:		// visitor interface
-	virtual void visit ( const TDLAxiomDeclaration& axiom ATTR_UNUSED ) { isLocal = true; }
+	virtual void visit ( const TDLAxiomDeclaration& ) { isLocal = true; }
 
 	virtual void visit ( const TDLAxiomEquivalentConcepts& axiom )
 	{
@@ -380,25 +474,31 @@ public:		// visitor interface
 	}
 	virtual void visit ( const TDLAxiomDisjointUnion& axiom )
 	{
+		// DisjointUnion(A, C1,..., Cn) is local if
+		//    (1) A and all of Ci are bot-equivalent,
+		// or (2) A and one Ci are top-equivalent and the remaining Cj are bot-equivalent
 		isLocal = false;
-		bool topLoc = topCLocal();
-		if ( !(topLoc ? isTopEquivalent(axiom.getC()) : isBotEquivalent(axiom.getC())) )
-			return;
+		bool lhsIsTopEq;
+		if ( isTopEquivalent(axiom.getC()) )
+			lhsIsTopEq = true;	// need to check (2)
+		else if ( isBotEquivalent(axiom.getC()) )
+			lhsIsTopEq = false;	// need to check (1)
+		else
+			return;				// neither (1) nor (2)
+
 		bool topEqDesc = false;
 		for ( TDLAxiomDisjointUnion::iterator p = axiom.begin(), p_end = axiom.end(); p != p_end; ++p )
 			if ( !isBotEquivalent(*p) )
 			{
-				if ( !topLoc )
-					return;	// non-local straight away
-				if ( isTopEquivalent(*p) )
+				if ( lhsIsTopEq && isTopEquivalent(*p) )
 				{
 					if ( topEqDesc )
-						return;	// 2nd top in there -- non-local
+						return;	// 2nd top in there -- violate (2) -- non-local
 					else
 						topEqDesc = true;
 				}
-				else
-					return;	// non-local
+				else	// either (1) or fail to have a top-eq for (2)
+					return;
 			}
 
 		isLocal = true;
@@ -463,53 +563,38 @@ public:		// visitor interface
 
 		isLocal = true;
 	}
-	virtual void visit ( const TDLAxiomSameIndividuals& axiom ATTR_UNUSED ) { isLocal = true; }
-	virtual void visit ( const TDLAxiomDifferentIndividuals& axiom ATTR_UNUSED ) { isLocal = true; }
+	virtual void visit ( const TDLAxiomSameIndividuals& ) { isLocal = false; }
+	virtual void visit ( const TDLAxiomDifferentIndividuals& ) { isLocal = false; }
 		/// there is no such axiom in OWL API, but I hope nobody would use Fairness here
-	virtual void visit ( const TDLAxiomFairnessConstraint& axiom ATTR_UNUSED ) { isLocal = true; }
+	virtual void visit ( const TDLAxiomFairnessConstraint& ) { isLocal = true; }
 
 	virtual void visit ( const TDLAxiomRoleInverse& axiom ) { isLocal = isREquivalent(axiom.getRole()) && isREquivalent(axiom.getInvRole()); }
-	virtual void visit ( const TDLAxiomORoleSubsumption& axiom ) { isLocal = isREquivalent(topRLocal() ? axiom.getRole() : axiom.getSubRole()); }
-	virtual void visit ( const TDLAxiomDRoleSubsumption& axiom ) { isLocal = isREquivalent(topRLocal() ? axiom.getRole() : axiom.getSubRole()); }
+	virtual void visit ( const TDLAxiomORoleSubsumption& axiom ) { isLocal = isTopEquivalent(axiom.getRole()) || isBotEquivalent(axiom.getSubRole()); }
+	virtual void visit ( const TDLAxiomDRoleSubsumption& axiom ) { isLocal = isTopEquivalent(axiom.getRole()) || isBotEquivalent(axiom.getSubRole()); }
 	virtual void visit ( const TDLAxiomORoleDomain& axiom )
-	{
-		isLocal = isTopEquivalent(axiom.getDomain());
-		if ( !topRLocal() )
-			isLocal |= isBotEquivalent(axiom.getRole());
-	}
+		{ isLocal = isTopEquivalent(axiom.getDomain()) || isBotEquivalent(axiom.getRole()); }
 	virtual void visit ( const TDLAxiomDRoleDomain& axiom )
-	{
-		isLocal = isTopEquivalent(axiom.getDomain());
-		if ( !topRLocal() )
-			isLocal |= isBotEquivalent(axiom.getRole());
-	}
+		{ isLocal = isTopEquivalent(axiom.getDomain()) || isBotEquivalent(axiom.getRole()); }
 	virtual void visit ( const TDLAxiomORoleRange& axiom )
-	{
-		isLocal = isTopEquivalent(axiom.getRange());
-		if ( !topRLocal() )
-			isLocal |= isBotEquivalent(axiom.getRole());
-	}
+		{ isLocal = isTopEquivalent(axiom.getRange()) || isBotEquivalent(axiom.getRole()); }
 	virtual void visit ( const TDLAxiomDRoleRange& axiom )
-	{
-		isLocal = isTopDT(axiom.getRange());
-		if ( !topRLocal() )
-			isLocal |= isBotEquivalent(axiom.getRole());
-	}
+		{ isLocal = isTopEquivalent(axiom.getRange()) || isBotEquivalent(axiom.getRole()); }
 	virtual void visit ( const TDLAxiomRoleTransitive& axiom ) { isLocal = isREquivalent(axiom.getRole()); }
-	virtual void visit ( const TDLAxiomRoleReflexive& axiom ) { isLocal = isREquivalent(axiom.getRole()); }
-	virtual void visit ( const TDLAxiomRoleIrreflexive& axiom ATTR_UNUSED ) { isLocal = !topRLocal(); }
+		/// as BotRole is irreflexive, the only local axiom is topEquivalent(R)
+	virtual void visit ( const TDLAxiomRoleReflexive& axiom ) { isLocal = isTopEquivalent(axiom.getRole()); }
+	virtual void visit ( const TDLAxiomRoleIrreflexive& axiom ) { isLocal = isBotEquivalent(axiom.getRole()); }
 	virtual void visit ( const TDLAxiomRoleSymmetric& axiom ) { isLocal = isREquivalent(axiom.getRole()); }
-	virtual void visit ( const TDLAxiomRoleAsymmetric& axiom ATTR_UNUSED ) { isLocal = !topRLocal(); }
-	virtual void visit ( const TDLAxiomORoleFunctional& axiom ) { isLocal = !topRLocal() && isBotEquivalent(axiom.getRole()); }
-	virtual void visit ( const TDLAxiomDRoleFunctional& axiom ) { isLocal = !topRLocal() && isBotEquivalent(axiom.getRole()); }
-	virtual void visit ( const TDLAxiomRoleInverseFunctional& axiom ) { isLocal = !topRLocal() && isBotEquivalent(axiom.getRole()); }
+	virtual void visit ( const TDLAxiomRoleAsymmetric& axiom ) { isLocal = isBotEquivalent(axiom.getRole()); }
+	virtual void visit ( const TDLAxiomORoleFunctional& axiom ) { isLocal = isBotEquivalent(axiom.getRole()); }
+	virtual void visit ( const TDLAxiomDRoleFunctional& axiom ) { isLocal = isBotEquivalent(axiom.getRole()); }
+	virtual void visit ( const TDLAxiomRoleInverseFunctional& axiom ) { isLocal = isBotEquivalent(axiom.getRole()); }
 
 	virtual void visit ( const TDLAxiomConceptInclusion& axiom ) { isLocal = isBotEquivalent(axiom.getSubC()) || isTopEquivalent(axiom.getSupC()); }
 	virtual void visit ( const TDLAxiomInstanceOf& axiom ) { isLocal = isTopEquivalent(axiom.getC()); }
-	virtual void visit ( const TDLAxiomRelatedTo& axiom ) { isLocal = topRLocal() && isTopEquivalent(axiom.getRelation()); }
-	virtual void visit ( const TDLAxiomRelatedToNot& axiom ) { isLocal = !topRLocal() && isBotEquivalent(axiom.getRelation()); }
-	virtual void visit ( const TDLAxiomValueOf& axiom ) { isLocal = topRLocal() && isTopEquivalent(axiom.getAttribute()); }
-	virtual void visit ( const TDLAxiomValueOfNot& axiom ) { isLocal = !topRLocal() && isBotEquivalent(axiom.getAttribute()); }
+	virtual void visit ( const TDLAxiomRelatedTo& axiom ) { isLocal = isTopEquivalent(axiom.getRelation()); }
+	virtual void visit ( const TDLAxiomRelatedToNot& axiom ) { isLocal = isBotEquivalent(axiom.getRelation()); }
+	virtual void visit ( const TDLAxiomValueOf& axiom ) { isLocal = isTopEquivalent(axiom.getAttribute()); }
+	virtual void visit ( const TDLAxiomValueOfNot& axiom ) { isLocal = isBotEquivalent(axiom.getAttribute()); }
 }; // SyntacticLocalityChecker
 
 #endif
