@@ -1,5 +1,5 @@
 /* This file is part of the FaCT++ DL reasoner
-Copyright (C) 2003-2011 by Dmitry Tsarkov
+Copyright (C) 2003-2012 by Dmitry Tsarkov
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -213,10 +213,8 @@ bool
 DlSatTester :: addSessionGCI ( BipolarPointer C, const DepSet& dep )
 {
 	SessionGCIs.push_back(C);
-	unsigned int n = 0;
-	DlCompletionTree* node = NULL;
-	while ( (node = CGraph.getNode(n++)) != NULL )
-		if ( !node->isDataNode() && addToDoEntry ( node, C, dep, "sg" ) )
+	for ( DlCompletionGraph::iterator p = CGraph.begin(), p_end = CGraph.end(); p != p_end; ++p )
+		if ( isNodeGloballyUsed(*p) && addToDoEntry ( *p, C, dep, "sg" ) )
 			return true;
 	return false;
 }
@@ -258,7 +256,7 @@ bool DlSatTester :: commonTacticBodySingleton ( const DLVertex& cur )
 bool DlSatTester :: commonTacticBodyAnd ( const DLVertex& cur )
 {
 #ifdef ENABLE_CHECKING
-	fpp_assert ( isPositive(curConcept.bp()) && ( cur.Type() == dtAnd || cur.Type() == dtCollection ) );	// safety check
+	fpp_assert ( isPositive(curConcept.bp()) && ( cur.Type() == dtAnd ) );	// safety check
 #endif
 
 	incStat(nAndCalls);
@@ -379,7 +377,7 @@ bool DlSatTester :: processOrEntry ( void )
 	// add new entry to current node; we know the result would be DONE
 	return
 #	ifdef RKG_USE_DYNAMIC_BACKJUMPING
-		addToDoEntry ( curNode, C, dep, reason );
+		addToDoEntry ( curNode, ConceptWDep(C,dep), reason );
 #	else
 		insertToDoEntry ( curNode, ConceptWDep(C,dep), DLHeap[C].Type(), reason );
 #	endif
@@ -407,6 +405,12 @@ bool DlSatTester :: commonTacticBodyAllComplex ( const DLVertex& cur )
 				switchResult ( addToDoEntry ( curNode, C+(*q)->final(), dep, "e" ) );
 		}
 
+	// apply all top-role transitions
+	if ( unlikely(RST.hasTopTransition()) )
+		for ( q = RST.begin(); q != end; ++q )
+			if ( (*q)->isTop() )
+				switchResult ( addSessionGCI ( C+(*q)->final(), dep ) );
+
 	// apply final-state rule
 	if ( state == 1 )
 		switchResult ( addToDoEntry ( curNode, cur.getC(), dep ) );
@@ -656,14 +660,11 @@ DlSatTester :: commonTacticBodySomeUniv ( const DLVertex& cur )
 
 	BipolarPointer C = inverse(cur.getC());
 	// check whether C is already in CGraph
-	unsigned int i = 0;;
-	DlCompletionTree* node;
-	while ( (node = CGraph.getNode(i++)) != NULL )
-		if ( node->label().contains(C) )
+	for ( DlCompletionGraph::iterator p = CGraph.begin(), p_end = CGraph.end(); p != p_end; ++p )
+		if ( isObjectNodeUnblocked(*p) && (*p)->label().contains(C) )
 			return false;
-	// make new node labelled with C
-	node = CGraph.getNewNode();
-	return initNewNode ( node, curConcept.getDep(), C );
+	// make new node labeled with C
+	return initNewNode ( CGraph.getNewNode(), curConcept.getDep(), C );
 }
 
 //-------------------------------------------------------------------------------
@@ -886,8 +887,13 @@ bool DlSatTester :: commonTacticBodyFunc ( const DLVertex& cur )	// for <=1 R co
 	fpp_assert ( isPositive(curConcept.bp()) && isFunctionalVertex(cur) );
 #endif
 
+	const TRole* R = cur.getRole();
+
+	if ( unlikely(R->isTop()) )
+		return processTopRoleFunc(cur);
+
 	// check whether we need to apply NN rule first
-	if ( isNNApplicable ( cur.getRole(), bpTOP, curConcept.bp()+1 ) )
+	if ( isNNApplicable ( R, bpTOP, curConcept.bp()+1 ) )
 		return commonTacticBodyNN(cur);	// after application func-rule would be checked again
 
 	incStat(nFuncCalls);
@@ -897,20 +903,20 @@ bool DlSatTester :: commonTacticBodyFunc ( const DLVertex& cur )	// for <=1 R co
 
 	// locate all R-neighbours of curNode
 	DepSet dummy;	// not used
-	findNeighbours ( cur.getRole(), bpTOP, dummy );
+	findNeighbours ( R, bpTOP, dummy );
 
 	// check if we have nodes to merge
 	if ( EdgesToMerge.size() < 2 )
 		return false;
 
 	// merge all nodes to the first (the least wrt nominal hierarchy) found node
-	EdgeVector::iterator q = EdgesToMerge.begin();
+	EdgeVector::iterator q = EdgesToMerge.begin(), q_end = EdgesToMerge.end();
 	DlCompletionTree* sample = (*q)->getArcEnd();
 	DepSet depF (curConcept.getDep());	// dep-set for merging
 	depF.add((*q)->getDep());
 
 	// merge all elements to sample (sample wouldn't be merge)
-	for ( ++q; q != EdgesToMerge.end(); ++q )
+	while ( ++q != q_end )
 		// during merge EdgesToMerge may became purged (see Nasty4) => check this
 		if ( !(*q)->getArcEnd()->isPBlocked() )
 			switchResult ( Merge ( (*q)->getArcEnd(), sample, depF+(*q)->getDep() ) );
@@ -926,77 +932,68 @@ bool DlSatTester :: commonTacticBodyLE ( const DLVertex& cur )	// for <=nR.C con
 #endif
 
 	incStat(nLeCalls);
-	BipolarPointer C = cur.getC();
 	const TRole* R = cur.getRole();
 
-	BCLE* bcLE = NULL;
+	if ( unlikely(R->isTop()) )
+		return processTopRoleLE(cur);
+
+	BipolarPointer C = cur.getC();
+	bool needInit = true;
 
 	if ( !isFirstBranchCall() )
 	{
 		if ( dynamic_cast<BCNN*>(bContext) != NULL )
 			return commonTacticBodyNN(cur);	// after application <=-rule would be checked again
-		if ( dynamic_cast<BCLE*>(bContext) != NULL )
-			goto applyLE;	// clash in LE-rule: skip all the rest
+		if ( dynamic_cast<BCLE<DlCompletionTreeArc>*>(bContext) != NULL )
+			needInit = false;	// clash in LE-rule: skip the initial checks
+		else	// the only possible case is choose-rule; in this case just continue
+			fpp_assert ( dynamic_cast<BCChoose*>(bContext) != NULL );
+	}
+	else	// if we are here that it IS first LE call
+		if ( isQuickClashLE(cur) )
+			return true;
 
-		// the only possible case is choose-rule; in this case just continue
-		fpp_assert ( dynamic_cast<BCChoose*>(bContext) != NULL );
+	// initial phase: choose-rule, NN-rule
+	if ( needInit )
+	{
+		// check if we have Qualified NR
+		if ( C != bpTOP )
+			switchResult ( commonTacticBodyChoose ( R, C ) );
+
+		// check whether we need to apply NN rule first
+		if ( isNNApplicable ( R, C, /*stopper=*/curConcept.bp()+cur.getNumberLE() ) )
+			return commonTacticBodyNN(cur);	// after application <=-rule would be checked again
 	}
 
-	// check if we have Qualified NR
-	if ( C != bpTOP )
-		switchResult ( commonTacticBodyChoose ( R, C ) );
-
-	// check whether we need to apply NN rule first
-	if ( isNNApplicable ( R, C, /*stopper=*/curConcept.bp()+cur.getNumberLE() ) )
-		return commonTacticBodyNN(cur);	// after application <=-rule would be checked again
-
-	// if we are here that it IS first LE call
-
-	if ( isQuickClashLE(cur) )
-		return true;
-
-	// we need to repeate merge until there will be necessary amount of edges
+	// we need to repeat merge until there will be necessary amount of edges
 	while (1)
 	{
 		if ( isFirstBranchCall() )
-		{
-			DepSet dep;
-			// check the amount of neighbours we have
-			findNeighbours ( R, C, dep );
-
-			// if the number of R-neighbours satisfies condition -- nothing to do
-			if ( EdgesToMerge.size() <= cur.getNumberLE() )
+			if ( initLEProcessing(cur) )
 				return false;
 
-			// init context
-			createBCLE();
-			bContext->branchDep += dep;
-
-			// setup BCLE
-			bcLE = static_cast<BCLE*>(bContext);
-
-			bcLE->EdgesToMerge.swap(EdgesToMerge);
-			bcLE->resetMCI();
-		}
-
-applyLE:	// skip init, because here we are after restoring
-
-		bcLE = static_cast<BCLE*>(bContext);
+		BCLE<DlCompletionTreeArc>* bcLE = static_cast<BCLE<DlCompletionTreeArc>*>(bContext);
 
 		if ( bcLE->noMoreLEOptions() )
-		{	// set global clashset to cummulative one from previous branch failures
+		{	// set global clashset to cumulative one from previous branch failures
 			useBranchDep();
 			return true;
 		}
 
 		// get from- and to-arcs using corresponding indexes in Edges
-		DlCompletionTreeArc* from = bcLE->getFrom();
-		DlCompletionTreeArc* to = bcLE->getTo();
+		DlCompletionTreeArc* fromArc = bcLE->getFrom();
+		DlCompletionTreeArc* toArc = bcLE->getTo();
+		DlCompletionTree* from = fromArc->getArcEnd();
+		DlCompletionTree* to = toArc->getArcEnd();
 
 		DepSet dep;	// empty dep-set
-		// fast check for from->end() and to->end() are in \neq
-		if ( CGraph.nonMergable ( from->getArcEnd(), to->getArcEnd(), dep ) )
+		// fast check for FROM =/= TO
+		if ( CGraph.nonMergable ( from, to, dep ) )
 		{
+			// need this for merging two nominal nodes
+			dep.add(fromArc->getDep());
+			dep.add(toArc->getDep());
+			// add dep-set from labels
 			if ( C == bpTOP )	// dep-set is known now
 				setClashSet(dep);
 			else	// QCR: update dep-set wrt C
@@ -1006,27 +1003,28 @@ applyLE:	// skip init, because here we are after restoring
 				bool test;
 
 				// here dep contains the clash-set
-				test = findConceptClash ( from->getArcEnd()->label().getLabel(tag), C, dep );
+				test = findConceptClash ( from->label().getLabel(tag), C, dep );
 				fpp_assert(test);
 				dep += getClashSet();	// save new dep-set
 
-				test = findConceptClash ( to->getArcEnd()->label().getLabel(tag), C, dep );
+				test = findConceptClash ( to->label().getLabel(tag), C, dep );
 				fpp_assert(test);
 				// both clash-sets are now in common clash-set
 			}
 
 			updateBranchDep();
 			bContext->nextOption();
-			goto applyLE;
+			fpp_assert(!isFirstBranchCall());
+			continue;
 		}
 
 		save();
 
 		// add depset from current level and FROM arc and to current dep.set
 		DepSet curDep(getCurDepSet());
-		curDep.add(from->getDep());
+		curDep.add(fromArc->getDep());
 
-		switchResult ( Merge ( from->getArcEnd(), to->getArcEnd(), curDep ) );
+		switchResult ( Merge ( from, to, curDep ) );
 		// it might be the case (see bIssue28) that after the merge there is an R-neigbour
 		// that have neither C or ~C in its label (it was far in the nominal cloud)
 		if ( C != bpTOP )
@@ -1034,6 +1032,30 @@ applyLE:	// skip init, because here we are after restoring
 	}
 }
 
+bool
+DlSatTester :: initLEProcessing ( const DLVertex& cur )
+{
+	DepSet dep;
+	// check the amount of neighbours we have
+	findNeighbours ( cur.getRole(), cur.getC(), dep );
+
+	// if the number of R-neighbours satisfies condition -- nothing to do
+	if ( EdgesToMerge.size() <= cur.getNumberLE() )
+		return true;
+
+	// init context
+	createBCLE();
+	bContext->branchDep += dep;
+
+	// setup BCLE
+	BCLE<DlCompletionTreeArc>* bcLE = static_cast<BCLE<DlCompletionTreeArc>*>(bContext);
+
+	bcLE->ItemsToMerge.swap(EdgesToMerge);
+	bcLE->resetMCI();
+	return false;
+}
+
+
 bool DlSatTester :: commonTacticBodyGE ( const DLVertex& cur )	// for >=nR.C concepts
 {
 #ifdef ENABLE_CHECKING
@@ -1044,15 +1066,186 @@ bool DlSatTester :: commonTacticBodyGE ( const DLVertex& cur )	// for >=nR.C con
 	if ( isCurNodeBlocked() )
 		return false;
 
+	const TRole* R = cur.getRole();
+
+	if ( unlikely(R->isTop()) )
+		return processTopRoleGE(cur);
+
 	incStat(nGeCalls);
 
 	if ( isQuickClashGE(cur) )
 		return true;
 
 	// create N new different edges
+	return createDifferentNeighbours ( R, cur.getC(), curConcept.getDep(), cur.getNumberGE(), BlockableLevel );
+}
+
+//-------------------------------------------------------------------------------
+//	Func/LE/GE with top role processing
+//-------------------------------------------------------------------------------
+
+bool
+DlSatTester :: processTopRoleFunc ( const DLVertex& cur )	// for <=1 R concepts
+{
+#ifdef ENABLE_CHECKING
+	fpp_assert ( isPositive(curConcept.bp()) && isFunctionalVertex(cur) );
+#endif
+
+	incStat(nFuncCalls);
+
+	if ( isQuickClashLE(cur) )
+		return true;
+
+	// locate all R-neighbours of curNode
+	DepSet dummy;	// not used
+	findCLabelledNodes ( bpTOP, dummy );
+
+	// check if we have nodes to merge
+	if ( NodesToMerge.size() < 2 )
+		return false;
+
+	// merge all nodes to the first (the least wrt nominal hierarchy) found node
+	NodeVector::iterator q = NodesToMerge.begin(), q_end = NodesToMerge.end();
+	DlCompletionTree* sample = *q;
+	DepSet dep (curConcept.getDep());	// dep-set for merging
+
+	// merge all elements to sample (sample wouldn't be merge)
+	while ( ++q != q_end )
+		// during merge EdgesToMerge may became purged (see Nasty4) => check this
+		if ( !(*q)->isPBlocked() )
+			switchResult ( Merge ( *q, sample, dep ) );
+
+	return false;
+}
+
+bool
+DlSatTester :: processTopRoleLE ( const DLVertex& cur )	// for <=nR.C concepts
+{
+#ifdef ENABLE_CHECKING
+	fpp_assert ( isPositive(curConcept.bp()) && ( cur.Type() == dtLE ) );
+#endif
+
+	BipolarPointer C = cur.getC();
+	bool needInit = true;
+
+	if ( !isFirstBranchCall() )
+	{
+		if ( dynamic_cast<BCLE<DlCompletionTree>*>(bContext) != NULL )
+			needInit = false;	// clash in LE-rule: skip the initial checks
+		else	// the only possible case is choose-rule; in this case just continue
+			fpp_assert ( dynamic_cast<BCChoose*>(bContext) != NULL );
+	}
+	else	// if we are here that it IS first LE call
+		if ( isQuickClashLE(cur) )
+			return true;
+
+	// initial phase: choose-rule, NN-rule
+	if ( needInit )
+	{
+		// check if we have Qualified NR
+		if ( C != bpTOP )
+			switchResult ( applyChooseRuleGlobally(C) );
+	}
+
+	// we need to repeat merge until there will be necessary amount of edges
+	while (1)
+	{
+		if ( isFirstBranchCall() )
+			if ( initTopLEProcessing(cur) )
+				return false;
+
+		BCLE<DlCompletionTree>* bcLE = static_cast<BCLE<DlCompletionTree>*>(bContext);
+
+		if ( bcLE->noMoreLEOptions() )
+		{	// set global clashset to cumulative one from previous branch failures
+			useBranchDep();
+			return true;
+		}
+
+		// get from- and to-arcs using corresponding indexes in Edges
+		DlCompletionTree* from = bcLE->getFrom();
+		DlCompletionTree* to = bcLE->getTo();
+
+		DepSet dep;	// empty dep-set
+		// fast check for FROM =/= TO
+		if ( CGraph.nonMergable ( from, to, dep ) )
+		{
+			// add dep-set from labels
+			if ( C == bpTOP )	// dep-set is known now
+				setClashSet(dep);
+			else	// QCR: update dep-set wrt C
+			{
+				// here we know that C is in both labels; set a proper clash-set
+				DagTag tag = DLHeap[C].Type();
+				bool test;
+
+				// here dep contains the clash-set
+				test = findConceptClash ( from->label().getLabel(tag), C, dep );
+				fpp_assert(test);
+				dep += getClashSet();	// save new dep-set
+
+				test = findConceptClash ( to->label().getLabel(tag), C, dep );
+				fpp_assert(test);
+				// both clash-sets are now in common clash-set
+			}
+
+			updateBranchDep();
+			bContext->nextOption();
+			fpp_assert(!isFirstBranchCall());
+			continue;
+		}
+
+		save();
+
+		// add depset from current level and FROM arc and to current dep.set
+		DepSet curDep(getCurDepSet());
+
+		switchResult ( Merge ( from, to, curDep ) );
+	}
+}
+
+bool
+DlSatTester :: processTopRoleGE ( const DLVertex& cur )	// for >=nR.C concepts
+{
+#ifdef ENABLE_CHECKING
+	fpp_assert ( isNegative(curConcept.bp()) && cur.Type() == dtLE );
+	fpp_assert ( !isCurNodeBlocked() );
+#endif
+
+	incStat(nGeCalls);
+
+	if ( isQuickClashGE(cur) )
+		return true;
+
+	// create N new different edges
+	// FIXME!! for now
 	return createDifferentNeighbours ( cur.getRole(), cur.getC(), curConcept.getDep(), cur.getNumberGE(), BlockableLevel );
 }
 
+bool
+DlSatTester :: initTopLEProcessing ( const DLVertex& cur )
+{
+	DepSet dep;
+	// check the amount of neighbours we have
+	findCLabelledNodes ( cur.getC(), dep );
+
+	// if the number of R-neighbours satisfies condition -- nothing to do
+	if ( NodesToMerge.size() <= cur.getNumberLE() )
+		return true;
+
+	// init context
+	createBCTopLE();
+	bContext->branchDep += dep;
+
+	// setup BCLE
+	BCLE<DlCompletionTree>* bcLE = static_cast<BCLE<DlCompletionTree>*>(bContext);
+
+	bcLE->ItemsToMerge.swap(NodesToMerge);
+	bcLE->resetMCI();
+	return false;
+}
+
+
 //-------------------------------------------------------------------------------
 //	Support for Func/LE/GE processing
 //-------------------------------------------------------------------------------
@@ -1268,14 +1461,34 @@ DlSatTester :: findNeighbours ( const TRole* Role, BipolarPointer C, DepSet& Dep
 	std::sort ( EdgesToMerge.begin(), EdgesToMerge.end(), EdgeCompare() );
 }
 
+class NodeCompare
+{
+public:
+	bool operator() ( DlCompletionTree* p, DlCompletionTree* q ) const { return *p < *q; }
+};
+
+void
+DlSatTester :: findCLabelledNodes ( BipolarPointer C, DepSet& Dep )
+{
+	NodesToMerge.clear();
+	DagTag tag = DLHeap[C].Type();
+
+	// FIXME!! do we need this for d-blocked nodes?
+	for ( DlCompletionGraph::iterator p = CGraph.begin(), p_end = CGraph.end(); p != p_end; ++p )
+		if ( isNodeGloballyUsed(*p) && findChooseRuleConcept ( (*p)->label().getLabel(tag), C, Dep ) )
+			NodesToMerge.push_back(*p);
+
+	// sort EdgesToMerge: From named nominals to generated nominals to blockable nodes
+	std::sort ( NodesToMerge.begin(), NodesToMerge.end(), NodeCompare() );
+}
+
+
 //-------------------------------------------------------------------------------
 //	Choose-rule processing
 //-------------------------------------------------------------------------------
 
 bool DlSatTester :: commonTacticBodyChoose ( const TRole* R, BipolarPointer C )
 {
-	DlCompletionTree::edge_iterator p;
-
 	// apply choose-rule for every R-neighbour
 	for ( DlCompletionTree::const_edge_iterator p = curNode->begin(), p_end = curNode->end(); p < p_end; ++p )
 		if ( (*p)->isNeighbour(R) )
@@ -1284,6 +1497,17 @@ bool DlSatTester :: commonTacticBodyChoose ( const TRole* R, BipolarPointer C )
 	return false;
 }
 
+/// apply choose-rule for all vertices (necessary for Top role in QCR)
+bool
+DlSatTester :: applyChooseRuleGlobally ( BipolarPointer C )
+{
+	for ( DlCompletionGraph::iterator p = CGraph.begin(), p_end = CGraph.end(); p != p_end; ++p )
+		if ( isObjectNodeUnblocked(*p) )	// FIXME!! think about d-blocked nodes
+			switchResult ( applyChooseRule ( *p, C ) );
+
+	return false;
+}
+
 //-------------------------------------------------------------------------------
 //	Support for choose-rule processing
 //-------------------------------------------------------------------------------
@@ -1334,7 +1558,7 @@ bool DlSatTester :: commonTacticBodyNN ( const DLVertex& cur )	// NN-rule
 	}
 
 	// take next NN number; save it as SAVE() will reset it to 0
-	unsigned int NN = bcNN->branchIndex;
+	unsigned int NN = bcNN->value;
 
 	// prepare to addition to the label
 	save();
