@@ -1,5 +1,5 @@
 /* This file is part of the FaCT++ DL reasoner
-Copyright (C) 2003-2011 by Dmitry Tsarkov
+Copyright (C) 2003-2012 by Dmitry Tsarkov
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -41,7 +41,7 @@ TBox :: TBox ( const ifOptionSet* Options, const std::string& TopORoleName, cons
 	, pOptions (Options)
 	, Status(kbLoading)
 	, curFeature(NULL)
-	, defConcept (NULL)
+	, pQuery(NULL)
 	, Concepts("concept")
 	, Individuals("individual")
 	, ORM ( /*data=*/false, TopORoleName, BotORoleName )
@@ -86,10 +86,7 @@ TBox :: ~TBox ( void )
 	delete pTop;
 	delete pBottom;
 	delete pTemp;
-
-	// delete the appropriate tax-vertex;
-	if ( defConcept != NULL )
-		clearQueryConcept();
+	delete pQuery;
 
 	// remove aux structures
 	delete stdReasoner;
@@ -155,11 +152,17 @@ void TBox :: initTopBottom ( void )
 	p->classTag = cttTrueCompletelyDefined;
 	pTop = p;
 
+	// "fresh" concept
 	p = new TConcept (" ");
 	p->setId(-1);
 	p->tsDepth=1;
 	p->classTag = cttTrueCompletelyDefined;
 	pTemp = p;
+
+	// query concept
+	p = new TConcept("FaCT++.default");
+	p->setSystem();
+	pQuery = p;
 }
 
 void TBox :: prepareReasoning ( void )
@@ -292,8 +295,7 @@ TBox :: isSatisfiable ( const TConcept* pConcept )
 	prepareFeatures ( pConcept, NULL );
 	bool result = getReasoner()->runSat ( pConcept->resolveId(), bpTOP );
 	// save cache
-	cache = getReasoner()->buildCacheByCGraph(result);
-	DLHeap.setCache ( pConcept->pName, cache );
+	DLHeap.setCache ( pConcept->pName, getReasoner()->buildCacheByCGraph(result) );
 	clearFeatures();
 
 	CHECK_LL_RETURN_VALUE(llSatResult,result);
@@ -451,24 +453,16 @@ void TBox :: readConfig ( const ifOptionSet* Options )
 TConcept*
 TBox :: createQueryConcept ( const DLTree* desc )
 {
-	static const char* const defConceptName = "FaCT++.default";
-
 	fpp_assert ( desc != NULL );
 
-	// we have to add this concept in any cases. So change undefined names mode
-	bool old = setForbidUndefinedNames(false);
-	defConcept = getConcept(defConceptName);
-	setForbidUndefinedNames(old);
-
-//	std::cerr << "Create new temp concept with description =" << desc << "\n";
-	fpp_assert ( defConcept != NULL );
-
+	// make sure that an old query is gone
+	clearQueryConcept();
 	// create description
-	deleteTree ( makeNonPrimitive ( defConcept, clone(desc) ) );
-	defConcept->setSystem();
-	defConcept->setIndex(nC-1);
+//	std::cerr << "Create new temp concept with description =" << desc << "\n";
+	deleteTree ( makeNonPrimitive ( pQuery, clone(desc) ) );
+	pQuery->setIndex(nC-1);
 
-	return defConcept;
+	return pQuery;
 }
 
 /// preprocess query concept: put description into DAG
@@ -495,16 +489,14 @@ void
 TBox :: classifyQueryConcept ( void )
 {
 	// prepare told subsumers for classification; as it is non-primitive, it is not CD
-	defConcept->initToldSubsumers();
-
-	fpp_assert ( pTax != NULL );
+	pQuery->initToldSubsumers();
 
 	// setup taxonomy behaviour flags
-	pTax->setCompletelyDefined ( false );	// non-primitive concept
-	pTax->setProgressIndicator(NULL);		// switch off the progress monitor
+	fpp_assert ( pTax != NULL );
+	pTax->setCompletelyDefined(false);	// non-primitive concept
 
 	// classify the concept
-	pTax->classifyEntry (defConcept);
+	pTax->classifyEntry(pQuery);
 }
 
 /// knowledge exploration: build a model and return a link to the root
