@@ -1,5 +1,5 @@
 /* This file is part of the FaCT++ DL reasoner
-Copyright (C) 2003-2011 by Dmitry Tsarkov
+Copyright (C) 2003-2012 by Dmitry Tsarkov
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -127,15 +127,17 @@ protected:	// types
 protected:	// typedefs
 		/// RW concept iterator
 	typedef ConceptCollection::iterator c_iterator;
-		/// RO concept iterator
-	typedef ConceptCollection::const_iterator c_const_iterator;
 		/// RW individual iterator
 	typedef IndividualCollection::iterator i_iterator;
-		/// RO individual iterator
-	typedef IndividualCollection::const_iterator i_const_iterator;
 		/// RO ExpressionArray iterator
 	typedef ExpressionArray::const_iterator ea_iterator;
 
+public:		// interface
+		/// RO concept iterator
+	typedef ConceptCollection::const_iterator c_const_iterator;
+		/// RO individual iterator
+	typedef IndividualCollection::const_iterator i_const_iterator;
+
 protected:	// members
 		/// relevance label (to determine logical features)
 	TLabeller relevance;
@@ -185,7 +187,7 @@ protected:	// members
 		/// concept representing temporary one that can not be used anywhere in the ontology
 	TConcept* pTemp;
 		/// temporary concept that represents query
-	TConcept* defConcept;
+	TConcept* pQuery;
 
 		/// all named concepts
 	ConceptCollection Concepts;
@@ -325,19 +327,11 @@ protected:	// methods
 	c_iterator c_begin ( void ) { return Concepts.begin(); }
 		/// RW end() for concepts
 	c_iterator c_end ( void ) { return Concepts.end(); }
-		/// RO begin() for concepts
-	c_const_iterator c_begin ( void ) const { return Concepts.begin(); }
-		/// RO end() for concepts
-	c_const_iterator c_end ( void ) const { return Concepts.end(); }
 
 		/// RW begin() for individuals
 	i_iterator i_begin ( void ) { return Individuals.begin(); }
 		/// RW end() for individuals
 	i_iterator i_end ( void ) { return Individuals.end(); }
-		/// RO begin() for individuals
-	i_const_iterator i_begin ( void ) const { return Individuals.begin(); }
-		/// RO end() for individuals
-	i_const_iterator i_end ( void ) const { return Individuals.end(); }
 
 //-----------------------------------------------------------------------------
 //--		internal ensure*-like interface
@@ -982,10 +976,10 @@ public:
 			return createTop();
 		if ( C == pBottom )
 			return createBottom();
-		return new DLTree ( TLexeme ( isIndividual(C) ? INAME : CNAME, C ) );
+		return createEntry ( isIndividual(C) ? INAME : CNAME, C );
 	}
 		/// get fresh concept
-	DLTree* getFreshConcept ( void ) const { return new DLTree ( TLexeme ( CNAME, pTemp ) ); }
+	DLTree* getFreshConcept ( void ) const { return createEntry ( CNAME, pTemp ); }
 
 	// n-ary absorption support
 
@@ -1106,6 +1100,14 @@ public:
 			return curFeature->hasSingletons();
 		else
 			return KBFeatures.hasSingletons();
+	}
+		/// check if the relevant part of KB contains top role
+	bool testHasTopRole ( void ) const
+	{
+		if ( curFeature != NULL )
+			return curFeature->hasTopRole();
+		else
+			return KBFeatures.hasTopRole();
 	}
 		/// check if Sorted Reasoning is applicable
 	bool canUseSortedReasoning ( void ) const
@@ -1118,6 +1120,20 @@ public:
 		/// @return true iff an individual blocks C deterministically
 	bool isBlockingDet ( TConcept* C ) const { return SameI.find(C)->second.second; }
 
+//-----------------------------------------------------------------------------
+//--		public iterators
+//-----------------------------------------------------------------------------
+
+		/// RO begin() for concepts
+	c_const_iterator c_begin ( void ) const { return Concepts.begin(); }
+		/// RO end() for concepts
+	c_const_iterator c_end ( void ) const { return Concepts.end(); }
+
+		/// RO begin() for individuals
+	i_const_iterator i_begin ( void ) const { return Individuals.begin(); }
+		/// RO end() for individuals
+	i_const_iterator i_end ( void ) const { return Individuals.end(); }
+
 //-----------------------------------------------------------------------------
 //--		public reasoning interface
 //-----------------------------------------------------------------------------
@@ -1151,7 +1167,7 @@ public:
 		/// classify query concept
 	void classifyQueryConcept ( void );
 		/// delete all query-related stuff
-	void clearQueryConcept ( void ) { DLHeap.removeQuery(Concepts); }
+	void clearQueryConcept ( void ) { DLHeap.removeQuery(); }
 
 //-----------------------------------------------------------------------------
 //--		public reasoning interface
