@@ -1,5 +1,5 @@
 /* This file is part of the FaCT++ DL reasoner
-Copyright (C) 2010-2011 by Dmitry Tsarkov
+Copyright (C) 2010-2012 by Dmitry Tsarkov
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -246,6 +246,7 @@ public:		// interface
 //	helper classes
 //------------------------------------------------------------------
 
+class TNamedEntry;
 
 //------------------------------------------------------------------
 ///	named entity
@@ -255,10 +256,12 @@ class TNamedEntity
 protected:	// members
 		/// name of the entity
 	std::string Name;
+		/// translated version of it
+	TNamedEntry* entry;
 
 public:		// interface
 		/// c'tor: initialise name
-	TNamedEntity ( const std::string& name ) : Name(name) {}
+	TNamedEntity ( const std::string& name ) : Name(name), entry(NULL) {}
 		/// empty d'tor
 	virtual ~TNamedEntity ( void ) {}
 
@@ -266,6 +269,11 @@ public:		// interface
 	const char* getName ( void ) const { return Name.c_str(); }
 		/// get access to the element itself
 	const TNamedEntity* getEntity ( void ) const { return this; }
+
+		/// set entry
+	void setEntry ( TNamedEntry* e ) { entry = e; }
+		/// get entry
+	TNamedEntry* getEntry ( void ) const { return entry; }
 }; // TNamedEntity
 
 //------------------------------------------------------------------
@@ -554,6 +562,16 @@ public:		// interface
 		/// empty d'tor
 	virtual ~TDLConceptAnd ( void ) {}
 
+		/// comparison for AND
+	bool operator == ( const TDLConceptAnd& expr ) const
+	{
+		if ( size() != expr.size() )
+			return false;
+		for ( iterator p = begin(), q = expr.begin(), p_end = end(); p != p_end; ++p, ++q )
+			if ( *p != *q )
+				return false;
+		return true;
+	}
 		/// accept method for the visitor pattern
 	void accept ( DLExpressionVisitor& visitor ) const { visitor.visit(*this); }
 }; // TDLConceptAnd
@@ -1513,7 +1531,7 @@ public:		// interface
 		/// empty c'tor
 	TDLDataType ( const TDLDataTypeExpression* type ) : Values(new DVCreator(type)) {}
 		/// empty d'tor
-	~TDLDataType ( void ) {}
+	virtual ~TDLDataType ( void ) {}
 
 		/// get new data value of the given type
 	const TDLDataValue* getValue ( const std::string& name ) { return Values.insert(name); }
@@ -1535,5 +1553,18 @@ public:		// interface
 	void accept ( DLExpressionVisitor& visitor ) const { visitor.visit(*this); }
 }; // TDLDataTypeName
 
+/// @return named data type that is the most basic datatype underlying TYPE
+inline TDLDataTypeName*
+getBasicDataType ( TDLDataTypeExpression* type )
+{
+	TDLDataTypeName* ret = dynamic_cast<TDLDataTypeName*>(type);
+	if ( ret == NULL )
+	{
+		TDLDataTypeRestriction* hostType = dynamic_cast<TDLDataTypeRestriction*>(type);
+		fpp_assert ( hostType != NULL );
+		ret = const_cast<TDLDataTypeName*>(hostType->getExpr());
+	}
+	return ret;
+}
 
 #endif
