@@ -1,5 +1,5 @@
 /* This file is part of the FaCT++ DL reasoner
-Copyright (C) 2003-2011 by Dmitry Tsarkov
+Copyright (C) 2003-2012 by Dmitry Tsarkov
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -146,8 +146,6 @@ protected:	// members
 protected:	// methods
 	// support for Anc/Desc filling and such
 
-		/// check if role or any of its sub-roles is transitive (ie non-simple)
-	void initSimple ( void );
 		/// check if the role is REALLY topmost-functional (internal-use only)
 	bool isRealTopFunc ( void ) const;
 		/// set up TopFunc member properly (internal-use only)
@@ -163,19 +161,14 @@ protected:	// methods
 		/// complete role automaton; keep track of processed roles in RINPROCESS
 	void completeAutomaton ( TRoleSet& RInProcess );
 		/// replace RoR [= R with Trans(R), replace synonyms in RS
-	void preprocessComposition ( TRoleVec& RS ) throw(EFPPCycleInRIA);
+	void preprocessComposition ( TRoleVec& RS );
 		/// add transition to automaton with the role
 	void addTrivialTransition ( const TRole* r )
 		{ A.addTransitionSafe ( A.initial(), new RATransition ( A.final(), r ) ); }
 		/// add automaton of a sub-role to a given one
 	void addSubRoleAutomaton ( const TRole* R )
 	{
-		if ( this == R )
-			return;
-
-		if ( R->isSimple() )
-			A.addSimpleRA(R->getAutomaton());
-		else
+		if ( this != R )	// non-trivial addition
 			A.addRA(R->getAutomaton());
 	}
 		/// get an automaton by a (possibly synonymical) role
@@ -235,12 +228,13 @@ public:		// interface
 
 	// different flags
 
-		/// register a Simple flag (not simple if role or any of its sub-roles is transitive)
-	FPP_ADD_FLAG(Simple,0x10);
 		/// distinguish data- and non-data role
-	FPP_ADD_FLAG(DataRole,0x20);
-		/// flag for recursive walks (used in Automaton creation)
-	FPP_ADD_FLAG(Finished,0x40);
+	FPP_ADD_FLAG(DataRole,0x10);
+
+	// simple
+
+		// @return true iff the role is simple
+	bool isSimple ( void ) const { return A.isSimple(); }
 
 	// functionality
 
@@ -379,7 +373,7 @@ public:		// interface
 		if ( !hasSpecialDomain() || getTRange() == NULL )
 			pSpecialDomain = createTop();
 		else
-			pSpecialDomain = createSNFForall ( new DLTree ( TLexeme ( RNAME, this ) ), clone(getTRange()) );
+			pSpecialDomain = createSNFForall ( createRole(this), clone(getTRange()) );
 	}
 		/// set the special domain value
 	void setSpecialDomain ( BipolarPointer bp ) { bpSpecialDomain = bp; }
@@ -480,7 +474,7 @@ public:		// interface
 		A.setup ( nRoles, isDataRole() );
 	}
 		/// check whether role description is consistent
-	void consistent ( void ) throw(EFPPNonSimpleRole)
+	void consistent ( void ) const
 	{
 		if ( isSimple() )		// all simple roles are consistent
 			return;
